"""Parse json config to swagger."""
import functools
import inspect
import json
from copy import deepcopy
from typing import Any, Callable, Optional, Set, Tuple, Union, cast

from rime_sdk.swagger.swagger_client import RegistryGenerativeLanguageModelInfo
from rime_sdk.swagger.swagger_client.models import (
    DataParamsFeatureIntersection,
    DataParamsRankingInfo,
    RegistryConnectionInfo,
    RegistryDatabricksInfo,
    RegistryDataCollectorInfo,
    RegistryDataFileInfo,
    RegistryDataInfo,
    RegistryDataLoadingInfo,
    RegistryDataParams,
    RegistryHuggingFaceDataInfo,
    RegistryHuggingFaceModelInfo,
    RegistryModelPathInfo,
    RegistryPredictionParams,
    RegistryPredInfo,
    RimeUUID,
    SchemaregistryModelInfo,
)

DEFAULT_DO_SAMPLING = True
CONNECTION_INFO_TYPE_SWAGGER = Union[
    RegistryDataFileInfo,
    RegistryDataLoadingInfo,
    RegistryDataCollectorInfo,
    RegistryDatabricksInfo,
    RegistryHuggingFaceDataInfo,
]
VALID_CONNECTION_TYPES = [
    "databricks",
    "data_file",
    "data_loading",
    "data_collector",
    "hugging_face",
]


def validate_types(func: Callable) -> Callable:
    """Wrap given function with a decorator that validates types of arguments."""

    @functools.wraps(func)
    def wrapper_validate(*args: Any, **kwargs: Any) -> Any:
        # https://docs.python.org/3.7/library/inspect.html#inspect.getfullargspec
        argspec = inspect.getfullargspec(func)
        for arg, argname in zip(args, argspec.args):
            typ = argspec.annotations[argname]
            if hasattr(typ, "__origin__"):
                # convert generic types to their origin, e.g. List[dict] -> list
                # (so typ can be used with isinstance)
                typ = typ.__origin__
            if not isinstance(arg, typ):
                raise TypeError(
                    f"Expected argument of type {typ} for parameter '{argname}'. "
                    f"Got {type(arg)}."
                )
        return func(*args, **kwargs)

    return wrapper_validate


# NOTE: whenever changing any of convert_single_pred_info_to_swagger,
# convert_single_data_info_to_swagger, convert_model_info_to_swagger, or any of their
# helper functions, be sure to copy those changes over to rime/core/config_parser.py.
# This is needed for the mock registry used in rime-engine ete tests.
@validate_types
def convert_pred_params_to_swagger(
    pred_params: dict,
) -> Optional[RegistryPredictionParams]:
    """Convert prediction params dictionary to swagger."""
    _config = deepcopy(pred_params)
    proto_names = RegistryPredictionParams.swagger_types
    param_config = {
        name: _config.pop(name) for name in proto_names if name in pred_params
    }
    if len(param_config) == 0:
        return None
    if _config:
        raise ValueError(
            f"Unknown prediction params: {list(_config)}"
            f"\nExpected: {list(proto_names)}"
        )
    return RegistryPredictionParams(**param_config)


@validate_types
def convert_data_params_to_swagger(
    data_params: dict,
) -> Optional[RegistryDataParams]:
    """Convert data params dictionary to swagger."""
    field_names = RegistryDataParams.swagger_types
    _config = deepcopy(data_params)
    param_config = {
        name: _config.pop(name) for name in field_names if name in data_params
    }
    if "sample" not in param_config:
        param_config["sample"] = DEFAULT_DO_SAMPLING
    if len(param_config) == 0:
        return None
    if _config:
        raise ValueError(
            "Found parameters in the data_params config that do"
            f" not belong: {list(_config)}"
            f"\nExpected: {list(field_names)}"
        )
    if "loading_kwargs" in param_config and param_config["loading_kwargs"] is not None:
        param_config["loading_kwargs"] = json.dumps(param_config["loading_kwargs"])
    if "ranking_info" in param_config and param_config["ranking_info"] is not None:
        param_config["ranking_info"] = DataParamsRankingInfo(
            **param_config["ranking_info"]
        )
    if "intersections" in param_config and param_config["intersections"] is not None:
        intersections = param_config["intersections"]
        param_config["intersections"] = [
            DataParamsFeatureIntersection(features=i.get("features", []))
            for i in intersections
        ]
    for param in ["text_features", "image_features"]:
        unstructured_feats = param_config.get(param)
        if unstructured_feats is not None and not isinstance(unstructured_feats, list):
            raise ValueError(
                f"`{param}` must be type `List[str]`. Got '{unstructured_feats}'."
            )
    return RegistryDataParams(**param_config)


@validate_types
def _mutate_data_file_conn_info_to_swag(
    connection_info: dict, path: str
) -> RegistryDataFileInfo:
    """Process data file connection info into a connection swagger."""
    required_keys = {"path"}
    _check_required_keys_exist(connection_info, required_keys, path)
    return RegistryDataFileInfo(
        path=connection_info.pop("path"),
        data_type=connection_info.pop("data_type", None),
    )


@validate_types
def _mutate_data_loader_conn_info_to_swag(
    connection_info: dict, path: str
) -> RegistryDataLoadingInfo:
    """Process data loader connection info into a connection swagger."""
    required_keys = {"path", "load_func_name"}
    _check_required_keys_exist(connection_info, required_keys, path)
    loader_kwargs_json = ""
    if "loader_kwargs" in connection_info and "loader_kwargs_json" in connection_info:
        raise ValueError(
            "Got both loader_kwargs and loader_kwargs_json, "
            "but only one should be provided."
        )
    elif "loader_kwargs" in connection_info:
        # This can be None, but we don't want to set, so check first.
        _val = connection_info.pop("loader_kwargs")
        if _val is not None:
            loader_kwargs_json = json.dumps(_val)
    elif "loader_kwargs_json" in connection_info:
        # This can be None, but we don't want to set, so check first.
        _val = connection_info.pop("loader_kwargs_json")
        if _val is not None:
            loader_kwargs_json = _val
    else:
        pass
    de_intg_id_str = connection_info.pop("data_endpoint_integration_id", None)
    return RegistryDataLoadingInfo(
        path=connection_info.pop("path"),
        load_func_name=connection_info.pop("load_func_name"),
        loader_kwargs_json=loader_kwargs_json,
        data_endpoint_integration_id=RimeUUID(uuid=de_intg_id_str)
        if de_intg_id_str
        else None,
    )


@validate_types
def _mutate_data_collector_conn_info_to_swag(
    connection_info: dict, path: str
) -> RegistryDataCollectorInfo:
    """Process data collector connection info into a connection info swagger."""
    required_keys = {"data_stream_id"}
    _check_required_keys_exist(connection_info, required_keys, path)
    data_stream_id = RimeUUID(connection_info.pop("data_stream_id"))
    return RegistryDataCollectorInfo(data_stream_id=data_stream_id)


@validate_types
def _mutate_delta_lake_conn_info_to_swag(
    connection_info: dict, path: str
) -> RegistryDatabricksInfo:
    """Process delta lake connection info into a connection info swagger."""
    required_keys = {"table_name"}
    _check_required_keys_exist(connection_info, required_keys, path)
    return RegistryDatabricksInfo(table_name=connection_info.pop("table_name"))


@validate_types
def _mutate_huggingface_conn_info_to_swag(
    connection_info: dict, path: str
) -> RegistryHuggingFaceDataInfo:
    """Process huggingface connection info into a connection info swagger."""
    required_keys = {"dataset_uri", "split_name"}
    _check_required_keys_exist(connection_info, required_keys, path)
    return RegistryHuggingFaceDataInfo(
        dataset_uri=connection_info.pop("dataset_uri"),
        split_name=connection_info.pop("split_name", None),
        loading_params_json=json.dumps(connection_info.pop("loading_params", None)),
    )


def _check_required_keys_exist(
    config: dict, required_keys: Set[str], path: str
) -> None:
    """Check that required keys exist in the configuration."""
    missing_keys = required_keys.difference(config)
    if missing_keys:
        raise ValueError(
            f"Missing arguments for {missing_keys} in {path}.\n"
            f"Expected: {list(required_keys)}\n"
            f"Got: {list(config.keys())}"
        )


def process_connection_info_to_swagger(
    connection_info: dict, config_type: str, path: str
) -> CONNECTION_INFO_TYPE_SWAGGER:
    """Process config connection info into a connection swagger and its swagger key."""
    _config = deepcopy(connection_info)
    _config = _config.pop(config_type)
    connection_loader_map = {
        "data_file": _mutate_data_file_conn_info_to_swag,
        "data_loading": _mutate_data_loader_conn_info_to_swag,
        "data_collector": _mutate_data_collector_conn_info_to_swag,
        "databricks": _mutate_delta_lake_conn_info_to_swag,
        "hugging_face": _mutate_huggingface_conn_info_to_swag,
    }
    _path = f"{path}.{config_type}"
    swagger = connection_loader_map[config_type](_config, _path)
    if _config:
        expected_field_names = getattr(swagger, "swagger_types", [])
        raise ValueError(
            f"Found parameters in the connection_info config of type {config_type}"
            f" that do not belong: {list(_config)}."
            f" Expected parameters: {expected_field_names}."
        )
    return cast(CONNECTION_INFO_TYPE_SWAGGER, swagger)


@validate_types
def convert_single_pred_info_to_swagger(
    pred_config: dict,
) -> Optional[RegistryPredInfo]:
    """Convert a dictionary to single pred info swagger message."""
    _config = deepcopy(pred_config)
    if "connection_info" not in _config:
        raise ValueError(
            "Missing required key 'connection_info' in prediction info config."
            f"\nGot: {list(_config)}."
        )
    connection_info = _config.pop("connection_info")
    pred_params_dict = _config.pop("pred_params", {})
    pred_params = convert_pred_params_to_swagger(pred_params_dict)
    connection_swagger, field = _process_connection_info_dict_swagger(
        connection_info, "pred_config"
    )
    connection_info = RegistryConnectionInfo()
    setattr(connection_info, field, connection_swagger)
    return RegistryPredInfo(pred_params=pred_params, connection_info=connection_info)


@validate_types
def _process_connection_info_dict_swagger(
    connection_info: dict, path: str
) -> Tuple[CONNECTION_INFO_TYPE_SWAGGER, str]:
    """Process the connection info dictionary."""
    if not connection_info:
        raise ValueError(f"No connection_info provided under {path}")
    config_type = list(connection_info.keys())[0]
    if len(connection_info) > 1:
        raise ValueError(
            f"Found parameters in the connection info config for {path} that do not"
            f" belong: {list(connection_info)}."
            f" Expected one of {VALID_CONNECTION_TYPES}."
        )
    if config_type not in VALID_CONNECTION_TYPES:
        raise ValueError(
            "Must specify connection type as part of `connection_info` in config. "
            f"Valid connection types are: {VALID_CONNECTION_TYPES}. Provided "
            f"connection_info of {path}.{connection_info}."
        )
    _path = f"{path}.connection_info"
    connection_swagger = process_connection_info_to_swagger(
        connection_info, config_type, _path
    )
    return connection_swagger, config_type


@validate_types
def convert_single_data_info_to_swagger(data_config: dict) -> RegistryDataInfo:
    """Convert a dictionary to a `DataInfo` Swagger message."""
    _config = deepcopy(data_config)
    if "connection_info" not in _config:
        raise ValueError(
            "Missing required key `connection_info` in data info config."
            f"\nGot: {list(_config)}"
        )
    connection_info = _config.pop("connection_info")
    path = "data_info"
    connection_swagger, field = _process_connection_info_dict_swagger(
        connection_info, path
    )
    params_dict = _config.pop("data_params", {})
    data_params = convert_data_params_to_swagger(params_dict)
    if _config:
        raise ValueError(
            f"Found parameters in the data info config for {path} that do not"
            f" belong: {list(_config)}."
            f" Expected parameters: {list(RegistryDataInfo.swagger_types)}."
        )
    connection_info = RegistryConnectionInfo()
    setattr(connection_info, field, connection_swagger)
    return RegistryDataInfo(
        data_params=data_params,
        connection_info=connection_info,
    )


@validate_types
def convert_model_info_to_swagger(model_config: dict) -> SchemaregistryModelInfo:
    """Convert a dictionary to model info swagger message."""
    _config = deepcopy(model_config)
    valid_model_infos = list(SchemaregistryModelInfo.swagger_types)
    if len(_config) != 1:
        raise ValueError(
            "Must specify exactly one valid model_info type in config. "
            f"Valid model_info types are: {valid_model_infos}. "
            f"Got: {list(_config)}."
        )
    model_type, model_info = next(iter(_config.items()))
    if not isinstance(model_info, dict):
        raise ValueError(
            f"model_info must be a dictionary. Got: {type(model_info)}."
            f"\nFull config: {model_config}"
        )
    try:
        if model_type == "model_path":
            model_info_swagger = SchemaregistryModelInfo(
                model_path=RegistryModelPathInfo(path=model_info.pop("path"))
            )
        elif model_type == "hugging_face":
            model_uri = model_info.pop("model_uri")
            config_d = model_info.pop("kwargs", {})
            if isinstance(config_d, dict):
                config_str = json.dumps(config_d)
            elif isinstance(config_d, str):
                config_str = config_d
            else:
                raise ValueError(
                    f"Invalid type for `kwargs` in hugging_face model_info."
                    f" Expected `dict` or json `str`. Got: {type(config_d)}"
                )
            model_info_swagger = SchemaregistryModelInfo(
                hugging_face=RegistryHuggingFaceModelInfo(
                    model_uri=model_uri, kwargs=config_str
                )
            )
        elif model_type == "generative_language_model":
            model_info_swagger = SchemaregistryModelInfo(
                generative_language_model=RegistryGenerativeLanguageModelInfo(
                    model_path=model_info.pop("model_path"),
                    system_prompt=model_info.pop("system_prompt", ""),
                )
            )
        else:
            raise ValueError(
                f"model_info type in config should be one of {valid_model_infos}. "
                f"Got {model_type}."
            )
    except KeyError as e:
        raise ValueError(f"Invalid config: {model_config}") from e

    if model_info:
        oneof_object = getattr(model_info_swagger, model_type)
        expected_field_names = getattr(oneof_object, "swagger_types", [])
        raise ValueError(
            f"Found parameters in the model_info config of type {model_type}"
            f" that do not belong: {list(model_info)}."
            f" Expected parameters: {expected_field_names}."
        )
    return model_info_swagger


def _get_uuid(id_str: str) -> dict:
    """Get a UUID from a string for validation.  For requests, use RimeUUID.

    Args:
        id_str: The string representation of the UUID.

    Returns:
        A dictionary with the UUID.
    """
    return {"uuid": id_str}


def _get_individual_tests_config_swagger(individual_tests_config: dict) -> str:
    try:
        return json.dumps(individual_tests_config)
    except TypeError:
        raise ValueError("The provided individual_tests_config was not valid JSON.")


def get_registry_ids(
    test_run_config: dict,
) -> Tuple[Optional[str], Optional[str], Optional[str]]:
    """Get model, ref and eval registry id from the config.

    Args:
        test_run_config: The test run configuration.
    """
    ref_dataset_id = test_run_config.get("data_info", {}).get("ref_dataset_id", None)
    eval_dataset_id = test_run_config.get("data_info", {}).get("eval_dataset_id", None)
    model_id = test_run_config.get("model_id", None)
    if not isinstance(model_id, str) and model_id is not None:
        model_id = model_id.get("uuid")
    return ref_dataset_id, eval_dataset_id, model_id


def validate_config(test_run_config: dict) -> None:
    """Validate the test run config.

    Args:
        test_run_config: The test run configuration.

    Raises:
        ValueError: If the test run configuration is invalid.
    """
    # TODO(VAL-2437): Add config validation service.
    if "model_id" in test_run_config:
        if isinstance(test_run_config["model_id"], str):
            test_run_config["model_id"] = _get_uuid(test_run_config["model_id"])
    if "run_time_info" in test_run_config:
        if "agent_id" in test_run_config["run_time_info"]:
            if isinstance(test_run_config["run_time_info"]["agent_id"], str):
                test_run_config["run_time_info"]["agent_id"] = _get_uuid(
                    test_run_config["run_time_info"]["agent_id"]
                )
    if (
        "test_suite_config" in test_run_config
        and "custom_tests" in test_run_config["test_suite_config"]
    ):
        if not isinstance(test_run_config["test_suite_config"]["custom_tests"], list):
            raise ValueError("The custom_tests configuration must be a list")
        test_run_config["test_suite_config"]["custom_tests"] = [
            json.dumps(test) if isinstance(test, dict) else test
            for test in test_run_config["test_suite_config"]["custom_tests"]
        ]
    if (
        "test_suite_config" in test_run_config
        and "individual_tests_config" in test_run_config["test_suite_config"]
        and isinstance(
            test_run_config["test_suite_config"]["individual_tests_config"], dict
        )
    ):
        test_run_config["test_suite_config"]["individual_tests_config"] = json.dumps(
            test_run_config["test_suite_config"]["individual_tests_config"]
        )



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RimeFileScanResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'result_update_time': 'datetime',
        'file_security_reports': 'list[RimeFileSecurityReport]',
        'scanned_file_paths': 'list[str]',
        'severity': 'LibgenerativeSeverity'
    }

    attribute_map = {
        'result_update_time': 'resultUpdateTime',
        'file_security_reports': 'fileSecurityReports',
        'scanned_file_paths': 'scannedFilePaths',
        'severity': 'severity'
    }

    def __init__(self, result_update_time=None, file_security_reports=None, scanned_file_paths=None, severity=None):  # noqa: E501
        """RimeFileScanResult - a model defined in Swagger"""  # noqa: E501
        self._result_update_time = None
        self._file_security_reports = None
        self._scanned_file_paths = None
        self._severity = None
        self.discriminator = None
        if result_update_time is not None:
            self.result_update_time = result_update_time
        if file_security_reports is not None:
            self.file_security_reports = file_security_reports
        if scanned_file_paths is not None:
            self.scanned_file_paths = scanned_file_paths
        if severity is not None:
            self.severity = severity

    @property
    def result_update_time(self):
        """Gets the result_update_time of this RimeFileScanResult.  # noqa: E501

        The time when the result was updated.  # noqa: E501

        :return: The result_update_time of this RimeFileScanResult.  # noqa: E501
        :rtype: datetime
        """
        return self._result_update_time

    @result_update_time.setter
    def result_update_time(self, result_update_time):
        """Sets the result_update_time of this RimeFileScanResult.

        The time when the result was updated.  # noqa: E501

        :param result_update_time: The result_update_time of this RimeFileScanResult.  # noqa: E501
        :type: datetime
        """

        self._result_update_time = result_update_time

    @property
    def file_security_reports(self):
        """Gets the file_security_reports of this RimeFileScanResult.  # noqa: E501

        The security reports for the files that were scanned.  # noqa: E501

        :return: The file_security_reports of this RimeFileScanResult.  # noqa: E501
        :rtype: list[RimeFileSecurityReport]
        """
        return self._file_security_reports

    @file_security_reports.setter
    def file_security_reports(self, file_security_reports):
        """Sets the file_security_reports of this RimeFileScanResult.

        The security reports for the files that were scanned.  # noqa: E501

        :param file_security_reports: The file_security_reports of this RimeFileScanResult.  # noqa: E501
        :type: list[RimeFileSecurityReport]
        """

        self._file_security_reports = file_security_reports

    @property
    def scanned_file_paths(self):
        """Gets the scanned_file_paths of this RimeFileScanResult.  # noqa: E501

        The list of files that were scanned.  # noqa: E501

        :return: The scanned_file_paths of this RimeFileScanResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._scanned_file_paths

    @scanned_file_paths.setter
    def scanned_file_paths(self, scanned_file_paths):
        """Sets the scanned_file_paths of this RimeFileScanResult.

        The list of files that were scanned.  # noqa: E501

        :param scanned_file_paths: The scanned_file_paths of this RimeFileScanResult.  # noqa: E501
        :type: list[str]
        """

        self._scanned_file_paths = scanned_file_paths

    @property
    def severity(self):
        """Gets the severity of this RimeFileScanResult.  # noqa: E501


        :return: The severity of this RimeFileScanResult.  # noqa: E501
        :rtype: LibgenerativeSeverity
        """
        return self._severity

    @severity.setter
    def severity(self, severity):
        """Sets the severity of this RimeFileScanResult.


        :param severity: The severity of this RimeFileScanResult.  # noqa: E501
        :type: LibgenerativeSeverity
        """

        self._severity = severity

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RimeFileScanResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RimeFileScanResult):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library that defines the interface to a Test Run object."""

from datetime import datetime
from typing import Iterator, Tuple

import pandas as pd
from deprecated import deprecated

from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.internal.swagger_parser import (
    parse_category_test_results,
    parse_test_case_result,
    parse_test_run_metadata,
)
from rime_sdk.internal.test_helpers import get_batch_result_response
from rime_sdk.internal.utils import convert_dict_to_html, make_link
from rime_sdk.swagger import swagger_client
from rime_sdk.swagger.swagger_client import ApiClient
from rime_sdk.swagger.swagger_client.models import TestrunresultGetTestRunResponse
from rime_sdk.test_batch import TestBatch


class TestRun:
    """An interface for a RIME Test Run object."""

    def __init__(self, api_client: ApiClient, test_run_id: str) -> None:
        """Create a new ``TestRun`` object.

        Arguments:
            api_client: ApiClient
                The client used to query about the status of the job.
            test_run_id: str
                The ID for the successfully completed Test Run.
        """
        self._api_client = api_client
        self._test_run_id = test_run_id

    @property
    def test_run_id(self) -> str:
        """Return the Test Run ID."""
        return self._test_run_id

    def __repr__(self) -> str:
        """Return a pretty-printed string representation of the object."""
        return f"TestRun(test_run_id={self.test_run_id})"

    def _repr_html_(self) -> str:
        """Return a pretty-printed HTML representation of the object."""
        info = {
            "Test Run ID": self._test_run_id,
            "Link": make_link(
                "https://" + self.get_link(), link_text="Test Run Result Page"
            ),
        }
        return convert_dict_to_html(info)

    def get_link(self) -> str:
        """Return the web app URL for the Test Run page.

        This page contains results for all Test Runs. To jump to the view which
        shows results for this specific Test Run, click on the corresponding time
        bin in the UI.
        """
        # Fetch Test Run metadata and return a dataframe of the single row.
        with RESTErrorHandler():
            api = swagger_client.ResultsReaderApi(self._api_client)
            res: TestrunresultGetTestRunResponse = api.results_reader_get_test_run(
                test_run_id=self.test_run_id
            )
        return res.test_run.web_app_url.url

    def get_result_df(self) -> pd.DataFrame:
        """Return high level summary information for a complete stress Test Run in a single-row dataframe.

        This dataframe includes information such as model metrics on the reference and\
        evaluation datasets, overall RIME results such as severity across tests,\
        and high level metadata such as the project ID and model task.

        Place these rows together to build a table of test to build a table of test
        run results for comparison. This only works for stress test jobs that
        have succeeded.

        Returns:
            pd.DataFrame:
                A `pandas.DataFrame` object containing the Test Run result.
                Use the `.columns` method on the returned dataframe to see what columns
                represent. Generally, these columns have information
                about the model and datasets as well as summary statistics such as the
                number of failing Test Cases or number of high severity Test Cases.

        Example:
            .. code-block:: python

                test_run = client.get_test_run(some_test_run_id)
                test_run_result_df = test_run.get_result_df()
        """
        api = swagger_client.ResultsReaderApi(self._api_client)
        # Fetch Test Run metadata and return a dataframe of the single row.
        with RESTErrorHandler():
            res: TestrunresultGetTestRunResponse = api.results_reader_get_test_run(
                test_run_id=self._test_run_id
            )
        # Use utility function for converting Protobuf to a dataframe.
        return parse_test_run_metadata(res.test_run)

    def get_test_cases_df(self, show_test_case_metrics: bool = False) -> pd.DataFrame:
        """Return all the Test Cases for a completed stress Test Run in a dataframe.

        This enables you to perform granular queries on Test Cases.
        For example, if you only care about subset performance tests and want to see
        the results on each feature, you can fetch all the Test Cases in a dataframe,
        then query on that dataframe by test type. This only works on stress test jobs
        that have succeeded.

        Arguments:
            show_test_case_metrics: bool = False
                Whether to show Test Case specific metrics. This could result in a
                sparse dataframe that is returned, since Test Cases return different
                metrics. Defaults to False.

        Returns:
            pd.DataFrame:
                A ``pandas.DataFrame`` object containing the Test Case results.
                Here is a selected list of columns in the output:

                    1. ``test_run_id``: ID of the parent Test Run.
                    2. ``features``: List of features that the Test Case ran on.
                    3. ``test_batch_type``: Type of test that was run (e.g. Subset AUC).
                    4. ``status``: Status of the Test Case (e.g. Pass, Fail, etc.).
                    5. ``severity``: Denotes the severity of the failure of the test.

        Example:
            .. code-block:: python

                # Wait until the job has finished, since this method only works on
                # SUCCEEDED jobs.
                job.get_status(verbose=True, wait_until_finish=True)
                # Get the Test Run result.
                test_run = job.get_test_run()
                # Dump the Test Cases in dataframe ``df``.
                df = test_run.get_test_cases_df()
        """
        api = swagger_client.ResultsReaderApi(self._api_client)
        all_test_cases = []
        # Iterate through the pages of Test Cases and break at the last page.
        page_token = ""
        while True:
            with RESTErrorHandler():
                if page_token == "":
                    res = api.results_reader_list_test_cases(
                        page_size=20,
                        list_test_cases_query_test_run_id=self._test_run_id,
                    )
                else:
                    res = api.results_reader_list_test_cases(
                        page_size=20,
                        page_token=page_token,
                    )
            if res.test_cases:
                tc_dicts = [
                    parse_test_case_result(tc, unpack_metrics=show_test_case_metrics)
                    for tc in res.test_cases
                ]
                # Concatenate the list of Test Case dictionaries.
                all_test_cases += tc_dicts
            # Advance to the next page of Test Cases.
            page_token = res.next_page_token
            # we've reached the last page of Test Cases.
            if not res.has_more:
                break

        return pd.DataFrame(all_test_cases)

    def get_test_batch(self, test_type: str) -> TestBatch:
        """Return the Test Batch object for the specified test type on this Test Run.

        A ``TestBatch`` object allows a user to query the results
        for the corresponding test. For example, the ``TestBatch``
        object representing ``subset_performance:subset_accuracy`` allows a user
        to understand the results of the ``subset_performance:subset_accuracy``
        test to varying levels of granularity.

        Args:
            test_type: str
                Name of the test. Structured as ``test_type:test_name``, e.g.
                ``subset_performance:subset_accuracy``.

        Returns:
            TestBatch:
                A ``TestBatch`` representing ``test_type``.

        Example:
            .. code-block:: python

                batch = test_run.get_test_batch("unseen_categorical")
        """
        test_batch_obj = TestBatch(self._api_client, self._test_run_id, test_type)
        # check that Test Batch exists by sending a request
        get_batch_result_response(self._test_run_id, test_type, self._api_client)
        return test_batch_obj

    def get_test_batches(self) -> Iterator[TestBatch]:
        """Return all ``TestBatch`` objects for the Test Run.

        Returns:
            Iterator[TestBatch]:
                An iterator of ``TestBatch`` objects.
        """
        api = swagger_client.ResultsReaderApi(self._api_client)
        # Iterate through the pages of Test Cases and break at the last page.
        page_token = ""
        while True:
            with RESTErrorHandler():
                if page_token == "":
                    res = api.results_reader_list_batch_results(
                        page_size=20,
                        test_run_id=self._test_run_id,
                    )
                else:
                    res = api.results_reader_list_batch_results(
                        page_size=20,
                        page_token=page_token,
                    )
            if res.test_batches:
                for test_batch in res.test_batches:
                    yield TestBatch(
                        self._api_client, self._test_run_id, test_batch.test_type
                    )
            # Advance to the next page of Test Cases.
            page_token = res.next_page_token
            # we've reached the last page of Test Cases.
            if not res.has_more:
                break

    def get_category_results_df(
        self,
        show_category_results_metrics: bool = False,
    ) -> pd.DataFrame:
        """Get all category results for a completed stress Test Run in a dataframe.

        This gives you the ability to perform granular queries on category tests.
        This only works on stress test jobs that have succeeded.

        Args:
            show_category_results_metrics: bool
               Boolean flag to request metrics related to the category results.
               Defaults to False.

        Returns:
            pd.DataFrame:
                A ``pandas.DataFrame`` object containing the Test Case results.
                Here is a selected list of columns in the output:
                1. ``id``: ID of the parent Test Run.
                2. ``name``: name of the category test.
                3. ``severity``: Denotes the severity of the failure of the test.
                4. ``test_batch_types``: List of tests that this category  uses.
                5. ``failing_test_types``: List of failing tests in this category.
                6. ``num_none_severity``: Count of tests with NONE severity.
                7. ``num_low_severity``: Count of tests with LOW severity.
                9. ``num_high_severity``: Count of tests with HIGH severity.

        Example:
            .. code-block:: python

                # Wait until the job has finished, since this method only works on
                # SUCCEEDED jobs.
                job.get_status(verbose=True, wait_until_finish=True)
                # Get the Test Run result.
                test_run = job.get_test_run()
                # Dump the Test Cases in dataframe ``df``.
                df = test_run.get_category_results_df()
        """
        api = swagger_client.ResultsReaderApi(self._api_client)
        all_category_tests = []
        with RESTErrorHandler():
            res = api.results_reader_get_category_results(self._test_run_id)
            if res.category_test_results:
                for category_test_result in res.category_test_results:
                    parsed_result = parse_category_test_results(
                        category_test_result,
                        unpack_metrics=show_category_results_metrics,
                    )
                    all_category_tests.append(parsed_result)
        return pd.DataFrame(all_category_tests)

    # TODO(RAT-2307): Remove this SDK
    @deprecated(
        "get_summary_tests_df has been deprecated in favor of get_category_results_df"
    )
    def get_summary_tests_df(
        self,
        show_summary_test_metrics: bool = False,
    ) -> pd.DataFrame:
        """Get summary tests for a completed stress Test Run in a dataframe.

        This gives you the ability to perform granular queries on summary tests.
        This only works on stress test jobs that have succeeded.

        Returns:
            pd.DataFrame:
                A ``pandas.DataFrame`` object containing the Test Case results.
                Here is a selected list of columns in the output:
                1. ``id``: ID of the parent Test Run.
                2. ``name``: name of the summary test.
                3. ``severity``: Denotes the severity of the failure of the test.
                4. ``test_batch_types``: List of tests that this summary tests uses.
                5. ``failing_test_types``: List of tests in this summary test that fail.
                6. ``num_none_severity``: Count of tests with NONE severity.
                7. ``num_low_severity``: Count of tests with LOW severity.
                9. ``num_high_severity``: Count of tests with HIGH severity.

        Example:
            .. code-block:: python

                # Wait until the job has finished, since this method only works on
                # SUCCEEDED jobs.
                job.get_status(verbose=True, wait_until_finish=True)
                # Get the Test Run result.
                test_run = job.get_test_run()
                # Dump the Test Cases in dataframe ``df``.
                df = test_run.get_summary_tests_df()
        """
        api = swagger_client.ResultsReaderApi(self._api_client)
        all_summary_tests = []
        # Iterate through the pages of Test Cases and break at the last page.
        page_token = ""
        while True:
            with RESTErrorHandler():
                if page_token == "":
                    res = api.results_reader_list_summary_tests(
                        page_size=20,
                        query_test_run_id=self._test_run_id,
                    )
                else:
                    res = api.results_reader_list_summary_tests(
                        page_size=20,
                        page_token=page_token,
                    )
            if res.summary_test_results:
                for summary_test_result in res.summary_test_results:
                    parsed_result = parse_category_test_results(
                        summary_test_result,
                        unpack_metrics=show_summary_test_metrics,
                    )
                    all_summary_tests.append(parsed_result)
            # Advance to the next page of Test Cases.
            page_token = res.next_page_token
            # we've reached the last page of Test Cases.
            if not res.has_more:
                break
        return pd.DataFrame(all_summary_tests)


class ContinuousTestRun(TestRun):
    """An interface for an individual RIME continuous Test Run."""

    def __init__(
        self,
        api_client: ApiClient,
        test_run_id: str,
        time_bin: Tuple[datetime, datetime],
    ) -> None:
        """Create a new ContinuousTestRun object.

        Arguments:
            api_client: ApiClient
                The client used to query about the status of the job.
            test_run_id: str
                The ID for the successfully completed Test Run.
            time_bin: Optional[Tuple[datetime, datetime]]
                A tuple of datetime objects indicating the start and end times.
        """
        super().__init__(api_client, test_run_id)
        self._time_bin = time_bin

    def _get_time_bin(self) -> Tuple[datetime, datetime]:
        """Return the time bin for this continuous Test Run."""
        return self._time_bin

    @property
    def start_time(self) -> datetime:
        """Return the start time."""
        return self._get_time_bin()[0]

    @property
    def end_time(self) -> datetime:
        """Return the end time."""
        return self._get_time_bin()[1]

    def get_link(self) -> str:
        """Return the web app URL which points to the Continuous Tests page.

        This page contains results for all Test Runs. To jump to the view which
        shows results for this specific Test Run, click on the corresponding time
        bin in the UI.

        Note: this is a string that should be copy-pasted into a browser.
        """
        # Fetch Test Run metadata and return a dataframe of the single row.
        with RESTErrorHandler():
            api = swagger_client.ResultsReaderApi(self._api_client)
            res: TestrunresultGetTestRunResponse = api.results_reader_get_test_run(
                test_run_id=self.test_run_id
            )
        # in CT we do not have unique URLs per Test Run so this link doesn't work
        # Maybe TODO: in the BE have the url attribute correspond to this directly
        invalid_url = res.test_run.web_app_url.url
        cutoff_loc = invalid_url.find("test-runs")
        valid_url = invalid_url[:cutoff_loc] + "ai-firewall/continuous-tests"
        return valid_url



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RimeStartValidatePredictionsTaskRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'project_id': 'RimeUUID',
        'dataset_id': 'str',
        'model_id': 'RimeUUID',
        'agent_id': 'RimeUUID'
    }

    attribute_map = {
        'project_id': 'projectId',
        'dataset_id': 'datasetId',
        'model_id': 'modelId',
        'agent_id': 'agentId'
    }

    def __init__(self, project_id=None, dataset_id=None, model_id=None, agent_id=None):  # noqa: E501
        """RimeStartValidatePredictionsTaskRequest - a model defined in Swagger"""  # noqa: E501
        self._project_id = None
        self._dataset_id = None
        self._model_id = None
        self._agent_id = None
        self.discriminator = None
        if project_id is not None:
            self.project_id = project_id
        self.dataset_id = dataset_id
        if model_id is not None:
            self.model_id = model_id
        if agent_id is not None:
            self.agent_id = agent_id

    @property
    def project_id(self):
        """Gets the project_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501


        :return: The project_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this RimeStartValidatePredictionsTaskRequest.


        :param project_id: The project_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :type: RimeUUID
        """

        self._project_id = project_id

    @property
    def dataset_id(self):
        """Gets the dataset_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501


        :return: The dataset_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :rtype: str
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this RimeStartValidatePredictionsTaskRequest.


        :param dataset_id: The dataset_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :type: str
        """
        if dataset_id is None:
            raise ValueError("Invalid value for `dataset_id`, must not be `None`")  # noqa: E501

        self._dataset_id = dataset_id

    @property
    def model_id(self):
        """Gets the model_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501


        :return: The model_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._model_id

    @model_id.setter
    def model_id(self, model_id):
        """Sets the model_id of this RimeStartValidatePredictionsTaskRequest.


        :param model_id: The model_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :type: RimeUUID
        """

        self._model_id = model_id

    @property
    def agent_id(self):
        """Gets the agent_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501


        :return: The agent_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :rtype: RimeUUID
        """
        return self._agent_id

    @agent_id.setter
    def agent_id(self, agent_id):
        """Sets the agent_id of this RimeStartValidatePredictionsTaskRequest.


        :param agent_id: The agent_id of this RimeStartValidatePredictionsTaskRequest.  # noqa: E501
        :type: RimeUUID
        """

        self._agent_id = agent_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RimeStartValidatePredictionsTaskRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RimeStartValidatePredictionsTaskRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GenerativefirewallRuleOutput(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'rule_name': 'str',
        'action': 'GenerativefirewallFirewallAction',
        'risk_category': 'RiskscoreRiskCategoryType',
        'pii_detection_details': 'GenerativefirewallPiiDetectionDetails',
        'language_detection_details': 'GenerativefirewallLanguageDetectionDetails',
        'prompt_injection_details': 'GenerativefirewallPromptInjectionDetails',
        'toxicity_detection_details': 'GenerativefirewallToxicityDetectionDetails',
        'code_detection_details': 'GenerativefirewallCodeDetectionDetails',
        'rule_eval_metadata': 'RuleOutputRuleEvaluationMetadata',
        'security_standards': 'list[ApigenerativefirewallStandardInfo]'
    }

    attribute_map = {
        'rule_name': 'ruleName',
        'action': 'action',
        'risk_category': 'riskCategory',
        'pii_detection_details': 'piiDetectionDetails',
        'language_detection_details': 'languageDetectionDetails',
        'prompt_injection_details': 'promptInjectionDetails',
        'toxicity_detection_details': 'toxicityDetectionDetails',
        'code_detection_details': 'codeDetectionDetails',
        'rule_eval_metadata': 'ruleEvalMetadata',
        'security_standards': 'securityStandards'
    }

    def __init__(self, rule_name=None, action=None, risk_category=None, pii_detection_details=None, language_detection_details=None, prompt_injection_details=None, toxicity_detection_details=None, code_detection_details=None, rule_eval_metadata=None, security_standards=None):  # noqa: E501
        """GenerativefirewallRuleOutput - a model defined in Swagger"""  # noqa: E501
        self._rule_name = None
        self._action = None
        self._risk_category = None
        self._pii_detection_details = None
        self._language_detection_details = None
        self._prompt_injection_details = None
        self._toxicity_detection_details = None
        self._code_detection_details = None
        self._rule_eval_metadata = None
        self._security_standards = None
        self.discriminator = None
        if rule_name is not None:
            self.rule_name = rule_name
        if action is not None:
            self.action = action
        if risk_category is not None:
            self.risk_category = risk_category
        if pii_detection_details is not None:
            self.pii_detection_details = pii_detection_details
        if language_detection_details is not None:
            self.language_detection_details = language_detection_details
        if prompt_injection_details is not None:
            self.prompt_injection_details = prompt_injection_details
        if toxicity_detection_details is not None:
            self.toxicity_detection_details = toxicity_detection_details
        if code_detection_details is not None:
            self.code_detection_details = code_detection_details
        if rule_eval_metadata is not None:
            self.rule_eval_metadata = rule_eval_metadata
        if security_standards is not None:
            self.security_standards = security_standards

    @property
    def rule_name(self):
        """Gets the rule_name of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The rule_name of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: str
        """
        return self._rule_name

    @rule_name.setter
    def rule_name(self, rule_name):
        """Sets the rule_name of this GenerativefirewallRuleOutput.


        :param rule_name: The rule_name of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: str
        """

        self._rule_name = rule_name

    @property
    def action(self):
        """Gets the action of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The action of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallFirewallAction
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this GenerativefirewallRuleOutput.


        :param action: The action of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallFirewallAction
        """

        self._action = action

    @property
    def risk_category(self):
        """Gets the risk_category of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The risk_category of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: RiskscoreRiskCategoryType
        """
        return self._risk_category

    @risk_category.setter
    def risk_category(self, risk_category):
        """Sets the risk_category of this GenerativefirewallRuleOutput.


        :param risk_category: The risk_category of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: RiskscoreRiskCategoryType
        """

        self._risk_category = risk_category

    @property
    def pii_detection_details(self):
        """Gets the pii_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The pii_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallPiiDetectionDetails
        """
        return self._pii_detection_details

    @pii_detection_details.setter
    def pii_detection_details(self, pii_detection_details):
        """Sets the pii_detection_details of this GenerativefirewallRuleOutput.


        :param pii_detection_details: The pii_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallPiiDetectionDetails
        """

        self._pii_detection_details = pii_detection_details

    @property
    def language_detection_details(self):
        """Gets the language_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The language_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallLanguageDetectionDetails
        """
        return self._language_detection_details

    @language_detection_details.setter
    def language_detection_details(self, language_detection_details):
        """Sets the language_detection_details of this GenerativefirewallRuleOutput.


        :param language_detection_details: The language_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallLanguageDetectionDetails
        """

        self._language_detection_details = language_detection_details

    @property
    def prompt_injection_details(self):
        """Gets the prompt_injection_details of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The prompt_injection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallPromptInjectionDetails
        """
        return self._prompt_injection_details

    @prompt_injection_details.setter
    def prompt_injection_details(self, prompt_injection_details):
        """Sets the prompt_injection_details of this GenerativefirewallRuleOutput.


        :param prompt_injection_details: The prompt_injection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallPromptInjectionDetails
        """

        self._prompt_injection_details = prompt_injection_details

    @property
    def toxicity_detection_details(self):
        """Gets the toxicity_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The toxicity_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallToxicityDetectionDetails
        """
        return self._toxicity_detection_details

    @toxicity_detection_details.setter
    def toxicity_detection_details(self, toxicity_detection_details):
        """Sets the toxicity_detection_details of this GenerativefirewallRuleOutput.


        :param toxicity_detection_details: The toxicity_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallToxicityDetectionDetails
        """

        self._toxicity_detection_details = toxicity_detection_details

    @property
    def code_detection_details(self):
        """Gets the code_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The code_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: GenerativefirewallCodeDetectionDetails
        """
        return self._code_detection_details

    @code_detection_details.setter
    def code_detection_details(self, code_detection_details):
        """Sets the code_detection_details of this GenerativefirewallRuleOutput.


        :param code_detection_details: The code_detection_details of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: GenerativefirewallCodeDetectionDetails
        """

        self._code_detection_details = code_detection_details

    @property
    def rule_eval_metadata(self):
        """Gets the rule_eval_metadata of this GenerativefirewallRuleOutput.  # noqa: E501


        :return: The rule_eval_metadata of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: RuleOutputRuleEvaluationMetadata
        """
        return self._rule_eval_metadata

    @rule_eval_metadata.setter
    def rule_eval_metadata(self, rule_eval_metadata):
        """Sets the rule_eval_metadata of this GenerativefirewallRuleOutput.


        :param rule_eval_metadata: The rule_eval_metadata of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: RuleOutputRuleEvaluationMetadata
        """

        self._rule_eval_metadata = rule_eval_metadata

    @property
    def security_standards(self):
        """Gets the security_standards of this GenerativefirewallRuleOutput.  # noqa: E501

        Standards encodes which regulatory standards (MITRE, OWASP) are addressed by this firewall rule.  # noqa: E501

        :return: The security_standards of this GenerativefirewallRuleOutput.  # noqa: E501
        :rtype: list[ApigenerativefirewallStandardInfo]
        """
        return self._security_standards

    @security_standards.setter
    def security_standards(self, security_standards):
        """Sets the security_standards of this GenerativefirewallRuleOutput.

        Standards encodes which regulatory standards (MITRE, OWASP) are addressed by this firewall rule.  # noqa: E501

        :param security_standards: The security_standards of this GenerativefirewallRuleOutput.  # noqa: E501
        :type: list[ApigenerativefirewallStandardInfo]
        """

        self._security_standards = security_standards

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GenerativefirewallRuleOutput, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GenerativefirewallRuleOutput):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Utility functions for converting between SDK args and proto objects."""

from datetime import datetime, timedelta
from typing import Any, Dict, List, Optional, TypeVar

from google.protobuf.json_format import MessageToDict
from google.protobuf.timestamp_pb2 import Timestamp

from rime_sdk.swagger.swagger_client import RimeUUID


def swagger_is_empty(swagger_val: Any) -> bool:
    """Check if a swagger object is empty."""
    return not bool(swagger_val)


TYPE_KEY = "enum_type"
PROTO_FIELD_KEY = "proto_field"
PROTO_TYPE_KEY = "proto_type"

BASE_TYPES = ["str", "float", "int", "bool"]

T = TypeVar("T")


def parse_dict_to_swagger(obj_dict: Optional[Dict], new_obj: T) -> T:
    """Parse non-nested dicts into a new object."""
    if obj_dict:
        for key, value in obj_dict.items():
            setattr(new_obj, key, value)
    return new_obj


def parse_str_to_uuid(uuid_str: Optional[str]) -> Optional[RimeUUID]:
    """Parse a string into a RimeUUID."""
    if uuid_str:
        return RimeUUID(uuid_str)
    return None


def serialize_datetime_to_proto_timestamp(date: datetime) -> Dict:
    """Convert datetime to swagger compatible grpc timestamp."""
    timestamp = Timestamp()
    timestamp.FromDatetime(date)
    # Swagger serialize datetime to iso8601 format, convert to
    # protobuf compatible serialization
    return MessageToDict(timestamp)


def rest_to_timedelta(delta: str) -> timedelta:
    """Convert a REST API compatible string to a time delta."""
    # REST API returns a string in seconds; e.g. one day is represented as "86400s"
    return timedelta(seconds=int(delta[:-1]))


def timedelta_to_rest(delta: timedelta) -> str:
    """Convert a time delta to a REST API compatible string."""
    return f"{int(delta.total_seconds())}s"


def select_oneof(oneof_map: Dict[str, Any], key_list: List[str]) -> Any:
    """Select one of the keys in the map.

    Args:
        oneof_map: The map to select from.
        key_list: The list of keys to select from.

    Returns:
        The key that was selected.

    Raises:
        ValueError
            When more than one of the keys are provided in the map.
    """
    selected_key = None
    for key in key_list:
        if key in oneof_map:
            if selected_key is not None:
                raise ValueError(
                    f"More than one of the keys {key_list} were provided in the map."
                )
            selected_key = key
    if selected_key is None:
        raise ValueError(f"None of the keys {key_list} were provided in the map.")
    return selected_key



"""Context manager to handle REST errors."""
import ast
from http import HTTPStatus
from types import TracebackType
from typing import Optional, Type

from rime_sdk.swagger.swagger_client.rest import ApiException


class RESTErrorHandler:
    """Class that can be used as a context manager.

    Used so that we handle REST errors in a uniform way in the code base.
    """

    def __init__(self, is_generative_firewall: bool = False) -> None:
        """Initializes the RESTErrorHandler."""
        if is_generative_firewall:
            self.http_unauthorized_error_message = (
                "{reason}: Your auth token is either invalid or expired. "
                "Please generate a new token from the Firewall Settings UI page. "
                "For more information on how to use auth tokens, please refer to the FirewallClient "
                "section of the Python SDK documentation."
            )
        else:
            self.http_unauthorized_error_message = (
                "{reason}: Your API token is either invalid or expired. "
                "Please generate a new API token from the API Access Tokens page under Workspace Settings."
            )

    def __enter__(self) -> None:
        """Needed for context manager usage."""

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Optional[bool]:
        """Handle exceptions in custom way on exit."""
        # Check if an exception was raised.
        if exc_type is not None:
            # For ApiException errors, we only raise the details.
            if isinstance(exc, ApiException):
                if exc.body:
                    # Authentication (401) errors need to be handled separately as they are nginx messages
                    if exc.status == HTTPStatus.UNAUTHORIZED:
                        raise ValueError(
                            self.http_unauthorized_error_message.format(
                                reason=exc.reason
                            )
                        ) from None
                    else:
                        body_string = exc.body.decode("UTF-8")
                        # Safely interpret string as a python dict to pull out message
                        body_dict = ast.literal_eval(body_string)
                        raise ValueError(
                            f"{exc.reason}: {body_dict['message']}"
                        ) from None
                else:
                    raise ValueError(exc.reason) from None
        # Returning None will raise any other error messages as they were.
        return None



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ValidateResponseProcessedRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'request_id': 'str',
        'user_input_text': 'str',
        'input_token_count': 'int',
        'contexts': 'list[str]',
        'contexts_token_count': 'int',
        'output_text': 'str',
        'output_token_count': 'int'
    }

    attribute_map = {
        'request_id': 'requestId',
        'user_input_text': 'userInputText',
        'input_token_count': 'inputTokenCount',
        'contexts': 'contexts',
        'contexts_token_count': 'contextsTokenCount',
        'output_text': 'outputText',
        'output_token_count': 'outputTokenCount'
    }

    def __init__(self, request_id=None, user_input_text=None, input_token_count=None, contexts=None, contexts_token_count=None, output_text=None, output_token_count=None):  # noqa: E501
        """ValidateResponseProcessedRequest - a model defined in Swagger"""  # noqa: E501
        self._request_id = None
        self._user_input_text = None
        self._input_token_count = None
        self._contexts = None
        self._contexts_token_count = None
        self._output_text = None
        self._output_token_count = None
        self.discriminator = None
        if request_id is not None:
            self.request_id = request_id
        if user_input_text is not None:
            self.user_input_text = user_input_text
        if input_token_count is not None:
            self.input_token_count = input_token_count
        if contexts is not None:
            self.contexts = contexts
        if contexts_token_count is not None:
            self.contexts_token_count = contexts_token_count
        if output_text is not None:
            self.output_text = output_text
        if output_token_count is not None:
            self.output_token_count = output_token_count

    @property
    def request_id(self):
        """Gets the request_id of this ValidateResponseProcessedRequest.  # noqa: E501

        Unique identifier of the request.  # noqa: E501

        :return: The request_id of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: str
        """
        return self._request_id

    @request_id.setter
    def request_id(self, request_id):
        """Sets the request_id of this ValidateResponseProcessedRequest.

        Unique identifier of the request.  # noqa: E501

        :param request_id: The request_id of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: str
        """

        self._request_id = request_id

    @property
    def user_input_text(self):
        """Gets the user_input_text of this ValidateResponseProcessedRequest.  # noqa: E501

        Information about the `user_input_text`. The raw input is truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :return: The user_input_text of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: str
        """
        return self._user_input_text

    @user_input_text.setter
    def user_input_text(self, user_input_text):
        """Sets the user_input_text of this ValidateResponseProcessedRequest.

        Information about the `user_input_text`. The raw input is truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :param user_input_text: The user_input_text of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: str
        """

        self._user_input_text = user_input_text

    @property
    def input_token_count(self):
        """Gets the input_token_count of this ValidateResponseProcessedRequest.  # noqa: E501


        :return: The input_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: int
        """
        return self._input_token_count

    @input_token_count.setter
    def input_token_count(self, input_token_count):
        """Sets the input_token_count of this ValidateResponseProcessedRequest.


        :param input_token_count: The input_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: int
        """

        self._input_token_count = input_token_count

    @property
    def contexts(self):
        """Gets the contexts of this ValidateResponseProcessedRequest.  # noqa: E501

        Information about the `contexts`. The contexts are truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :return: The contexts of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._contexts

    @contexts.setter
    def contexts(self, contexts):
        """Sets the contexts of this ValidateResponseProcessedRequest.

        Information about the `contexts`. The contexts are truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :param contexts: The contexts of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: list[str]
        """

        self._contexts = contexts

    @property
    def contexts_token_count(self):
        """Gets the contexts_token_count of this ValidateResponseProcessedRequest.  # noqa: E501


        :return: The contexts_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: int
        """
        return self._contexts_token_count

    @contexts_token_count.setter
    def contexts_token_count(self, contexts_token_count):
        """Sets the contexts_token_count of this ValidateResponseProcessedRequest.


        :param contexts_token_count: The contexts_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: int
        """

        self._contexts_token_count = contexts_token_count

    @property
    def output_text(self):
        """Gets the output_text of this ValidateResponseProcessedRequest.  # noqa: E501

        Information about the `output_text`. The output text is truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :return: The output_text of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: str
        """
        return self._output_text

    @output_text.setter
    def output_text(self, output_text):
        """Sets the output_text of this ValidateResponseProcessedRequest.

        Information about the `output_text`. The output text is truncated if it exceeds a certain token length so we do not denial of service downstream logging and data systems.  # noqa: E501

        :param output_text: The output_text of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: str
        """

        self._output_text = output_text

    @property
    def output_token_count(self):
        """Gets the output_token_count of this ValidateResponseProcessedRequest.  # noqa: E501


        :return: The output_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :rtype: int
        """
        return self._output_token_count

    @output_token_count.setter
    def output_token_count(self, output_token_count):
        """Sets the output_token_count of this ValidateResponseProcessedRequest.


        :param output_token_count: The output_token_count of this ValidateResponseProcessedRequest.  # noqa: E501
        :type: int
        """

        self._output_token_count = output_token_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ValidateResponseProcessedRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ValidateResponseProcessedRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Set of constants such as user-facing strings."""
from rime_sdk.swagger.swagger_client.models import (
    MonitorMonitorType,
    RiskscoreRiskCategoryType,
)

# User-facing strings for Monitor types.
MONITOR_TYPE_DEFAULT_STR: str = "Default"
MONITOR_TYPE_CUSTOM_STR: str = "Custom"
MONITOR_TYPE_TO_SWAGGER = {
    MONITOR_TYPE_DEFAULT_STR: MonitorMonitorType.DEFAULT,
    MONITOR_TYPE_CUSTOM_STR: MonitorMonitorType.CUSTOM,
}

# User-facing strings for Risk Categories.
RISK_CATEGORY_OPERATIONAL_STR: str = "Operational"
RISK_CATEGORY_BIAS_AND_FAIRNESS_STR: str = "Bias_and_Fairness"
RISK_CATEGORY_SECURITY_STR: str = "Security"
RISK_CATEGORY_CUSTOM_STR: str = "Custom"
RISK_CATEGORY_TO_SWAGGER = {
    RISK_CATEGORY_OPERATIONAL_STR: RiskscoreRiskCategoryType.OPERATIONAL_HEALTH,
    RISK_CATEGORY_BIAS_AND_FAIRNESS_STR: RiskscoreRiskCategoryType.ETHICAL_RISK,
    RISK_CATEGORY_SECURITY_STR: RiskscoreRiskCategoryType.SECURITY_RISK,
    RISK_CATEGORY_CUSTOM_STR: RiskscoreRiskCategoryType.CUSTOM,
}



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class TestrunTestSuiteConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'global_test_sensitivity': 'TestrunTestSensitivity',
        'individual_tests_config': 'str',
        'custom_tests': 'list[str]',
        'global_exclude_columns': 'list[str]'
    }

    attribute_map = {
        'global_test_sensitivity': 'globalTestSensitivity',
        'individual_tests_config': 'individualTestsConfig',
        'custom_tests': 'customTests',
        'global_exclude_columns': 'globalExcludeColumns'
    }

    def __init__(self, global_test_sensitivity=None, individual_tests_config=None, custom_tests=None, global_exclude_columns=None):  # noqa: E501
        """TestrunTestSuiteConfig - a model defined in Swagger"""  # noqa: E501
        self._global_test_sensitivity = None
        self._individual_tests_config = None
        self._custom_tests = None
        self._global_exclude_columns = None
        self.discriminator = None
        if global_test_sensitivity is not None:
            self.global_test_sensitivity = global_test_sensitivity
        if individual_tests_config is not None:
            self.individual_tests_config = individual_tests_config
        if custom_tests is not None:
            self.custom_tests = custom_tests
        if global_exclude_columns is not None:
            self.global_exclude_columns = global_exclude_columns

    @property
    def global_test_sensitivity(self):
        """Gets the global_test_sensitivity of this TestrunTestSuiteConfig.  # noqa: E501


        :return: The global_test_sensitivity of this TestrunTestSuiteConfig.  # noqa: E501
        :rtype: TestrunTestSensitivity
        """
        return self._global_test_sensitivity

    @global_test_sensitivity.setter
    def global_test_sensitivity(self, global_test_sensitivity):
        """Sets the global_test_sensitivity of this TestrunTestSuiteConfig.


        :param global_test_sensitivity: The global_test_sensitivity of this TestrunTestSuiteConfig.  # noqa: E501
        :type: TestrunTestSensitivity
        """

        self._global_test_sensitivity = global_test_sensitivity

    @property
    def individual_tests_config(self):
        """Gets the individual_tests_config of this TestrunTestSuiteConfig.  # noqa: E501

        A serialized JSON blob including a config for each test.  # noqa: E501

        :return: The individual_tests_config of this TestrunTestSuiteConfig.  # noqa: E501
        :rtype: str
        """
        return self._individual_tests_config

    @individual_tests_config.setter
    def individual_tests_config(self, individual_tests_config):
        """Sets the individual_tests_config of this TestrunTestSuiteConfig.

        A serialized JSON blob including a config for each test.  # noqa: E501

        :param individual_tests_config: The individual_tests_config of this TestrunTestSuiteConfig.  # noqa: E501
        :type: str
        """

        self._individual_tests_config = individual_tests_config

    @property
    def custom_tests(self):
        """Gets the custom_tests of this TestrunTestSuiteConfig.  # noqa: E501

        A list of JSON custom test configurations.  # noqa: E501

        :return: The custom_tests of this TestrunTestSuiteConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._custom_tests

    @custom_tests.setter
    def custom_tests(self, custom_tests):
        """Sets the custom_tests of this TestrunTestSuiteConfig.

        A list of JSON custom test configurations.  # noqa: E501

        :param custom_tests: The custom_tests of this TestrunTestSuiteConfig.  # noqa: E501
        :type: list[str]
        """

        self._custom_tests = custom_tests

    @property
    def global_exclude_columns(self):
        """Gets the global_exclude_columns of this TestrunTestSuiteConfig.  # noqa: E501

        Features to exclude from all tests.  # noqa: E501

        :return: The global_exclude_columns of this TestrunTestSuiteConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._global_exclude_columns

    @global_exclude_columns.setter
    def global_exclude_columns(self, global_exclude_columns):
        """Sets the global_exclude_columns of this TestrunTestSuiteConfig.

        Features to exclude from all tests.  # noqa: E501

        :param global_exclude_columns: The global_exclude_columns of this TestrunTestSuiteConfig.  # noqa: E501
        :type: list[str]
        """

        self._global_exclude_columns = global_exclude_columns

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TestrunTestSuiteConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TestrunTestSuiteConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library defining the interface to the generative firewall."""
import atexit
import importlib
import time
from typing import Any, Iterable, List, Optional

from urllib3.util import Retry

from rime_sdk.authenticator import Authenticator
from rime_sdk.client import RETRY_HTTP_STATUS
from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.internal.utils import is_positive_number, remove_null_values_from_dict
from rime_sdk.swagger.swagger_client import (
    ApiClient,
    ApigenerativefirewallCreateFirewallInstanceRequest,
    ApigenerativefirewallCreateFirewallInstanceResponse,
    ApigenerativefirewallListFirewallInstancesResponse,
    Configuration,
    FirewallApi,
    FirewallinstanceFirewallInstanceFirewallInstanceIdUuidBody,
    FirewallInstanceIdUuidValidateBody,
    FirewallInstanceManagerApi,
    GenerativefirewallFirewallInstanceInfo,
    GenerativefirewallFirewallInstanceStatus,
    GenerativefirewallFirewallRuleConfig,
    GenerativefirewallGetFirewallEffectiveConfigResponse,
    GenerativefirewallGetFirewallInstanceResponse,
    RimeUUID,
)
from rime_sdk.swagger.swagger_client.models.generativefirewall_firewall_rule_type import (
    GenerativefirewallFirewallRuleType,
)
from rime_sdk.swagger.swagger_client.models.generativefirewall_individual_rules_config import (
    GenerativefirewallIndividualRulesConfig,
)
from rime_sdk.swagger.swagger_client.models.rime_language import RimeLanguage

_DEFAULT_CHANNEL_TIMEOUT = 60.0


VALID_LANGUAGES = [
    RimeLanguage.EN,
    RimeLanguage.JA,
]

_DEFAULT_BLOCK_UNTIL_READY_TIMEOUT_SEC = 300.0


def _get_firewall_instance_info(
    instance_manager_client: FirewallInstanceManagerApi, firewall_instance_id: str
) -> GenerativefirewallFirewallInstanceInfo:
    with RESTErrorHandler(is_generative_firewall=True):
        res: GenerativefirewallGetFirewallInstanceResponse = (
            instance_manager_client.get_instance(firewall_instance_id)
        )
        fw_instance: GenerativefirewallFirewallInstanceInfo = res.firewall_instance
    return fw_instance


def _validate_individual_rules_config(individual_rules_config: Any) -> None:
    """Validate the raw individual rules config provided by the user.

    Raises an exception if the configuration is malformed.
    """
    if not isinstance(individual_rules_config, dict):
        raise TypeError("individual rules config must be a dict")
    swagger_module = importlib.import_module("rime_sdk.swagger.swagger_client")
    for k, v in individual_rules_config.items():
        if k not in GenerativefirewallIndividualRulesConfig.swagger_types:
            raise ValueError(
                f"Found unrecognized rule '{k}' in `rule_config.individual_rules_config`. "
                f"The list of accepted individual rule configs is: {list(GenerativefirewallIndividualRulesConfig.swagger_types.keys())}"
            )
        # Dynamically load the Python autogenerated Swagger class from the
        # top-level Swagger module, which makes all models accessible.
        # This is only janky because each `swagger_types` stores the string
        # name of the class but not a reference to the class itself.
        desired_class = getattr(
            swagger_module,
            GenerativefirewallIndividualRulesConfig.swagger_types[k],
        )
        swagger_types: dict = desired_class.swagger_types  # type: ignore[attr-defined]
        if not isinstance(v, dict):
            raise TypeError(
                "Each individual rule config must be a dictionary. "
                f"Value '{v}' for rule config '{k}' is not a dictionary"
            )
        unrecognized_keys = set(v.keys()) - set(swagger_types.keys())
        if len(unrecognized_keys) > 0:
            keys_str = ", ".join(unrecognized_keys)
            raise ValueError(
                f"Provided individual rule config for rule '{k}' has unrecognized keys '{keys_str}'. "
                f"Desired structure for rule '{k}': {swagger_types}"
            )


def _validate_selected_rules(selected_rules: Any) -> None:
    """Validate the selected rules in a user's rule config."""
    if not isinstance(selected_rules, list):
        raise TypeError("selected_rules must be a list")

    valid_enum_values = [
        getattr(GenerativefirewallFirewallRuleType, attr)
        for attr in dir(GenerativefirewallFirewallRuleType)
        if attr.isupper() and not attr == "UNSPECIFIED"
    ]

    diff = set(selected_rules) - set(valid_enum_values)
    if len(diff) > 0:
        diff_str = ", ".join(diff)
        valid_enum_values_str = ", ".join(valid_enum_values)
        raise ValueError(
            f"Unrecognized firewall rule type enum values '{diff_str}'. "
            f"Valid firewall rule types are '{valid_enum_values_str}'."
        )


def _get_validated_firewall_config(
    config: dict,
) -> GenerativefirewallFirewallRuleConfig:
    _config = config.copy()
    language = _config.pop("language", None)
    if language is not None and language not in VALID_LANGUAGES:
        raise ValueError(
            f"Provided language {language} is invalid, please choose one of the "
            f"following values {VALID_LANGUAGES}"
        )

    individual_rules_config = _config.pop("individual_rules_config", None)
    if individual_rules_config is not None:
        _validate_individual_rules_config(individual_rules_config)

    selected_rules = _config.pop("selected_rules", None)
    if selected_rules is not None:
        _validate_selected_rules(selected_rules)

    if _config:
        raise ValueError(
            f"Found unexpected keys in `rule_config`: {list(_config.keys())}"
        )
    return GenerativefirewallFirewallRuleConfig(
        individual_rules_config=individual_rules_config,
        selected_rules=selected_rules,
        language=language,
    )


def _validate_block_until_ready_params(
    block_until_ready: bool = True,
    block_until_ready_timeout_sec: Optional[float] = None,
    block_until_ready_poll_rate_sec: Optional[float] = None,
    block_until_consecutive_ready_count: Optional[int] = None,
) -> None:
    if block_until_ready:
        if block_until_ready_timeout_sec is not None and not is_positive_number(
            block_until_ready_timeout_sec
        ):
            raise ValueError(
                "Invalid value for 'block_until_ready_timeout_sec'"
                f": {block_until_ready_timeout_sec}. The value must be a positive number."
            )
        if block_until_ready_poll_rate_sec is not None and not is_positive_number(
            block_until_ready_poll_rate_sec
        ):
            raise ValueError(
                "Invalid value for 'block_until_ready_poll_rate_sec'"
                f": {block_until_ready_poll_rate_sec}. The value must be a positive number."
            )
        if block_until_consecutive_ready_count is not None and not is_positive_number(
            block_until_consecutive_ready_count
        ):
            raise ValueError(
                "Invalid value for 'block_until_consecutive_ready_count'"
                f": {block_until_consecutive_ready_count}. The value must be a positive number."
            )


class FirewallInstance:
    """An interface to a single instance of the firewall running on a cluster.

    Each FirewallInstance has its own rule configuration and can be accessed
    by its unique ID.
    This allows users to customize the behavior of the firewall for different
    use cases.
    Note: FirewallInstance should not be instantiated directly, but instead
    instantiated through methods of the FirewallClient.

    Args:
        firewall_instance_id: str
            The unique ID of the FirewallInstance.
        api_client: ApiClient
            API client for interacting with the cluster.
    """

    def __init__(
        self,
        firewall_instance_id: str,
        api_client: ApiClient,
    ) -> None:
        """Initialize a new FirewallInstance."""
        self._firewall_instance_id = firewall_instance_id

        self._api_client = api_client
        self._firewall_client = FirewallApi(self._api_client)
        self._instance_manager_client = FirewallInstanceManagerApi(self._api_client)

    def validate(
        self,
        user_input_text: Optional[str] = None,
        output_text: Optional[str] = None,
        contexts: Optional[List[str]] = None,
    ) -> dict:
        """Validate model input and/or output text."""
        if user_input_text is None and output_text is None and contexts is None:
            raise ValueError(
                "Must provide either input text, output text, or context documents to validate."
            )

        body = FirewallInstanceIdUuidValidateBody(
            user_input_text=user_input_text,
            contexts=contexts,
            output_text=output_text,
            firewall_instance_id=RimeUUID(uuid=self.firewall_instance_id),
        )
        with RESTErrorHandler(is_generative_firewall=True):
            response = self._firewall_client.validate(
                body=body, firewall_instance_id_uuid=self.firewall_instance_id
            )
        res_dict = response.to_dict()
        remove_null_values_from_dict(res_dict)
        return res_dict

    def get_effective_config(self) -> dict:
        """Get the effective configuration for the FirewallInstance.

        This effective configuration has default values filled in and shows what
        is actually being used at runtime.
        """
        with RESTErrorHandler(is_generative_firewall=True):
            res: GenerativefirewallGetFirewallEffectiveConfigResponse = (
                self._firewall_client.effective_config(
                    firewall_instance_id_uuid=self.firewall_instance_id
                )
            )
        if res.config is None:
            return {}
        return res.config.to_dict()

    def block_until_ready(
        self,
        verbose: bool = True,
        timeout_sec: float = _DEFAULT_BLOCK_UNTIL_READY_TIMEOUT_SEC,
        poll_rate_sec: float = 5.0,
        consecutive_ready_count: int = 1,
    ) -> None:
        """Block until ready blocks until the FirewallInstance is ready.

        Raises:
            TimeoutError
                This error is raised if the FirewallInstance is not ready by the deadline
                set through `timeout_sec`.
        """
        if not is_positive_number(timeout_sec):
            raise ValueError(
                f"Invalid value for 'timeout_sec': {timeout_sec}. "
                f"The value must be a positive number."
            )
        if not is_positive_number(poll_rate_sec):
            raise ValueError(
                f"Invalid value for 'poll_rate_sec': {poll_rate_sec}. "
                f"The value must be a positive number."
            )
        if not is_positive_number(consecutive_ready_count):
            raise ValueError(
                f"Invalid value for 'consecutive_ready_count': {consecutive_ready_count}. "
                f"The value must be a positive number."
            )
        start_time = time.time()
        if verbose:
            print(
                "Waiting until FirewallInstance {} is ready with timeout {}s".format(
                    self.firewall_instance_id,
                    timeout_sec,
                )
            )
        ready_count = 0
        while True:
            status = self.status

            # Get the total time the caller has been waiting and print it out.
            cur_time = time.time()
            elapsed_time = cur_time - start_time
            if verbose:
                minute, second = divmod(elapsed_time, 60)
                print(
                    "\rStatus: {}, Time Elapsed: {:02}:{:05.2f}".format(
                        status,
                        int(minute),
                        second,
                    ),
                    end="",
                )

            if status == GenerativefirewallFirewallInstanceStatus.READY:
                ready_count += 1
            else:
                ready_count = 0
            # Either the firewall reaches the ready status or we are ready to
            # time out.
            if ready_count >= consecutive_ready_count or elapsed_time >= timeout_sec:
                break

            time.sleep(poll_rate_sec)

        if verbose:
            # Print an extra line because the status has a carriage return.
            print()

        if status != GenerativefirewallFirewallInstanceStatus.READY:
            raise TimeoutError(
                f"FirewallInstance did not reach status READY by the timeout {timeout_sec}s"
            )

    def update_firewall_instance(
        self,
        config: Optional[dict] = None,
        description: Optional[str] = None,
        block_until_ready: bool = True,
        block_until_ready_verbose: Optional[bool] = None,
        block_until_ready_timeout_sec: Optional[float] = None,
        block_until_ready_poll_rate_sec: Optional[float] = None,
        block_until_consecutive_ready_count: Optional[int] = 2,
    ) -> None:
        """Update the config or description of the FirewallInstance.

        Args:
            config: str
                New config for the FirewallInstance.

            description: str
                New description for the FirewallInstance.

            block_until_ready: bool = True
                Whether to block until the FirewallInstance is ready.

            block_until_ready_verbose: Optional[bool] = None
                Whether to print out information while waiting for the FirewallInstance to come up.

            block_until_ready_timeout_sec: Optional[float] = None
                How many seconds to wait until the FirewallInstance comes up before timing out.

            block_until_ready_poll_rate_sec: Optional[float] = None
                How often to poll the FirewallInstance status.

            block_until_consecutive_ready_count: Optional[int] = 2
                Number of consecutive READY status poll to wait for, before returning.
        """
        if config is None and description is None:
            raise ValueError("A new `config` or `description` must be provided.")
        _validate_block_until_ready_params(
            block_until_ready,
            block_until_ready_timeout_sec,
            block_until_ready_poll_rate_sec,
            block_until_consecutive_ready_count,
        )
        final_conf = (
            _get_validated_firewall_config(config) if config is not None else None
        )
        with RESTErrorHandler(is_generative_firewall=True):
            body = FirewallinstanceFirewallInstanceFirewallInstanceIdUuidBody(
                config=final_conf,
                description=description,
            )
            self._instance_manager_client.update_instance(
                body,
                self.firewall_instance_id,
            )

        if block_until_ready:
            # Forward keyword arguments to the FirewallInstance.block_until_ready call.
            block_until_ready_kwargs: dict[str, Any] = {}
            if block_until_ready_verbose:
                block_until_ready_kwargs["verbose"] = block_until_ready_verbose
            if block_until_ready_timeout_sec:
                block_until_ready_kwargs["timeout_sec"] = block_until_ready_timeout_sec
            if block_until_ready_poll_rate_sec:
                block_until_ready_kwargs[
                    "poll_rate_sec"
                ] = block_until_ready_poll_rate_sec
            if block_until_consecutive_ready_count:
                block_until_ready_kwargs[
                    "consecutive_ready_count"
                ] = block_until_consecutive_ready_count
            self.block_until_ready(**block_until_ready_kwargs)

    @property
    def rule_config(self) -> dict:
        """Access the rule config of the FirewallInstance.

        This config is immutable after it is created.
        """
        fw_instance = _get_firewall_instance_info(
            self._instance_manager_client,
            self.firewall_instance_id,
        )
        conf = fw_instance.config
        return conf.to_dict() if conf is not None else {}

    @property
    def firewall_instance_id(self) -> str:
        """Access the UUID of the FirewallInstance."""
        return self._firewall_instance_id

    @property
    def status(self) -> str:
        """Access the current status of the FirewallInstance."""
        fw_instance = _get_firewall_instance_info(
            self._instance_manager_client,
            self.firewall_instance_id,
        )
        return fw_instance.deployment_status

    @property
    def description(self) -> str:
        """Access the description of the FirewallInstance."""
        fw_instance = _get_firewall_instance_info(
            self._instance_manager_client,
            self.firewall_instance_id,
        )
        return fw_instance.description if fw_instance.description else ""

    def __str__(self) -> str:
        """String representation of the FirewallInstance."""
        return f'FirewallInstance(id="{self.firewall_instance_id}", description="{self.description}")'


class FirewallClient:
    """An interface to connect to FirewallInstances on a firewall cluster.

    Create a firewall instance by specifying the rule configuration.
    It will take anywhere from a few seconds to a few minutes to spin up, but
    once it is ready, it can respond to validation requests with the custom
    configuration.
    A single firewall cluster can have many firewall instances.
    They are independent from each other.

    Args:
        domain: str
            The base domain/address of the firewall.
        api_key: str
            The API key is used to authenticate to the firewall. If an api_key is
            provided, you do not need to provide an auth_token.
        auth_token: str
            The auth token is generated in the Firewall UI and is used to authenticate
            to the firewall. If the auth_token is provided, you do not need to provide
            an api_key. Auth tokens are only available when the Firewall UI has been
            enabled.
        channel_timeout: float
            The amount of time in seconds to wait for responses from the firewall.
    """

    def __init__(
        self,
        domain: str,
        api_key: str = "",
        auth_token: str = "",
        channel_timeout: float = _DEFAULT_CHANNEL_TIMEOUT,
    ):
        """Create a new Client connected to the services available at `domain`."""
        configuration = Configuration()
        configuration.api_key["X-Firewall-Api-Key"] = api_key
        configuration.api_key["X-Firewall-Auth-Token"] = auth_token
        if domain.endswith("/"):
            domain = domain[:-1]
        if not domain.startswith("https://") and not domain.startswith("http://"):
            domain = "https://" + domain
        configuration.host = domain
        self._api_client = ApiClient(configuration)
        # Prevent race condition in pool.close() triggered by swagger generated code
        atexit.register(self._api_client.pool.close)
        # Sets the timeout and hardcoded retries parameter for the api client.
        self._api_client.rest_client.pool_manager.connection_pool_kw[
            "timeout"
        ] = channel_timeout
        # Configure the retries against the firewall cluster.
        # Due to NGINX refresh issues breaking connections, we want to allow
        # retries on `read` errors on all idempotent HTTP methods.
        # The current implementation of PATCH with gRPC-gateway proto field
        # masks is idempotent, so it can be safely retried.
        # Note: the Retry will only retry on read errors if the HTTP method
        # belongs to one of the configured `allowed_methods`.
        self._api_client.rest_client.pool_manager.connection_pool_kw["retries"] = Retry(
            read=3,
            status=3,
            status_forcelist=RETRY_HTTP_STATUS,
            allowed_methods=Retry.DEFAULT_ALLOWED_METHODS.union(["PATCH"]),
            backoff_factor=0.5,
        )
        self._instance_manager_client = FirewallInstanceManagerApi(self._api_client)

    def login(self, email: str, system_account: bool = False) -> None:
        """Login to obtain an auth token.

        Args:
            email: str
                The user's email address that is used to authenticate.

            system_account: bool
                This flag specifies whether it is for a system account token or not.

        Example:
             .. code-block:: python

                firewall.login("dev@robustintelligence.com", True)
        """
        authenticator = Authenticator()
        authenticator.auth(self._api_client.configuration.host, email, system_account)
        with open("./token.txt", "r+") as file1:
            self._api_client.configuration.api_key[
                "X-Firewall-Auth-Token"
            ] = file1.read()

    def list_firewall_instances(self) -> Iterable[FirewallInstance]:
        """List the FirewallInstances for the given cluster."""
        with RESTErrorHandler(is_generative_firewall=True):
            res: ApigenerativefirewallListFirewallInstancesResponse = (
                self._instance_manager_client.list_instances()
            )
            firewall_instances: List[
                GenerativefirewallFirewallInstanceInfo
            ] = res.firewall_instances
            for fwinfo in firewall_instances:
                yield FirewallInstance(
                    fwinfo.firewall_instance_id.uuid, self._api_client
                )

    def create_firewall_instance(
        self,
        rule_config: dict,
        description: str = "",
        block_until_ready: bool = True,
        block_until_ready_verbose: Optional[bool] = None,
        block_until_ready_timeout_sec: Optional[float] = None,
        block_until_ready_poll_rate_sec: Optional[float] = None,
    ) -> FirewallInstance:
        """Create a FirewallInstance with the specified rule configuration.

        This method blocks until the FirewallInstance is ready.

        Args:
            rule_config: dict
                Dictionary containing the rule config to customize the behavior
                of the FirewallInstance.

            description: str = ""
                Human-readable description of the FirewallInstance.

            block_until_ready: bool = True
                Whether to block until the FirewallInstance is ready.

            block_until_ready_verbose: Optional[bool] = None
                Whether to print out information while waiting for the FirewallInstance to come up.

            block_until_ready_timeout_sec: Optional[float] = None
                How many seconds to wait until the FirewallInstance comes up before timing out.

            block_until_ready_poll_rate_sec: Optional[float] = None
                How often to poll the FirewallInstance status.

        Returns:
            FirewallInstance that is ready to accept validation requests.

        Raises:
            TimeoutError
                This error is generated if the FirewallInstance is not ready by the deadline
                set through `timeout_sec`.
        """
        _validate_block_until_ready_params(
            block_until_ready,
            block_until_ready_timeout_sec,
            block_until_ready_poll_rate_sec,
        )
        final_conf = _get_validated_firewall_config(rule_config)
        req = ApigenerativefirewallCreateFirewallInstanceRequest(
            config=final_conf, description=description
        )
        with RESTErrorHandler(is_generative_firewall=True):
            res: ApigenerativefirewallCreateFirewallInstanceResponse = (
                self._instance_manager_client.create_instance(req)
            )
        fw = FirewallInstance(res.firewall_instance_id.uuid, self._api_client)

        if block_until_ready:
            # Forward keyword arguments to the FirewallInstance.block_until_ready call.
            block_until_ready_kwargs: dict[str, Any] = {}
            if block_until_ready_verbose:
                block_until_ready_kwargs["verbose"] = block_until_ready_verbose
            if block_until_ready_timeout_sec:
                block_until_ready_kwargs["timeout_sec"] = block_until_ready_timeout_sec
            if block_until_ready_poll_rate_sec:
                block_until_ready_kwargs[
                    "poll_rate_sec"
                ] = block_until_ready_poll_rate_sec
            fw.block_until_ready(**block_until_ready_kwargs)

        return fw

    def get_firewall_instance(self, firewall_instance_id: str) -> FirewallInstance:
        """Get a FirewallInstance from the cluster.

        Args:
            firewall_instance_id: str
                The UUID string of the FirewallInstance to retrieve.

        Returns:
            FirewallInstance:
                A firewall instance on which to perform validation.

        """
        with RESTErrorHandler(is_generative_firewall=True):
            fwinfo = _get_firewall_instance_info(
                self._instance_manager_client, firewall_instance_id
            )
        return FirewallInstance(
            firewall_instance_id=fwinfo.firewall_instance_id.uuid,
            api_client=self._api_client,
        )

    def delete_firewall_instance(self, firewall_instance_id: str) -> None:
        """Delete a FirewallInstance from the cluster.

        Careful when deleting a FirewallInstance: in-flight validation traffic
        will be interrupted.

        Args:
            firewall_instance_id: str
                The UUID string of the FirewallInstance to hard delete.
        """
        with RESTErrorHandler(is_generative_firewall=True):
            self._instance_manager_client.delete_instance(firewall_instance_id)



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RimeListImagesRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'page_token': 'str',
        'page_size': 'str',
        'pip_libraries': 'list[ListImagesRequestPipLibraryFilter]'
    }

    attribute_map = {
        'page_token': 'pageToken',
        'page_size': 'pageSize',
        'pip_libraries': 'pipLibraries'
    }

    def __init__(self, page_token=None, page_size=None, pip_libraries=None):  # noqa: E501
        """RimeListImagesRequest - a model defined in Swagger"""  # noqa: E501
        self._page_token = None
        self._page_size = None
        self._pip_libraries = None
        self.discriminator = None
        if page_token is not None:
            self.page_token = page_token
        if page_size is not None:
            self.page_size = page_size
        if pip_libraries is not None:
            self.pip_libraries = pip_libraries

    @property
    def page_token(self):
        """Gets the page_token of this RimeListImagesRequest.  # noqa: E501

        Specifies a page of the list returned by a ListImages query. The ListImages query returns a pageToken when there is more than one page of results.  # noqa: E501

        :return: The page_token of this RimeListImagesRequest.  # noqa: E501
        :rtype: str
        """
        return self._page_token

    @page_token.setter
    def page_token(self, page_token):
        """Sets the page_token of this RimeListImagesRequest.

        Specifies a page of the list returned by a ListImages query. The ListImages query returns a pageToken when there is more than one page of results.  # noqa: E501

        :param page_token: The page_token of this RimeListImagesRequest.  # noqa: E501
        :type: str
        """

        self._page_token = page_token

    @property
    def page_size(self):
        """Gets the page_size of this RimeListImagesRequest.  # noqa: E501

        The maximum number of Image objects to return in a single page.  # noqa: E501

        :return: The page_size of this RimeListImagesRequest.  # noqa: E501
        :rtype: str
        """
        return self._page_size

    @page_size.setter
    def page_size(self, page_size):
        """Sets the page_size of this RimeListImagesRequest.

        The maximum number of Image objects to return in a single page.  # noqa: E501

        :param page_size: The page_size of this RimeListImagesRequest.  # noqa: E501
        :type: str
        """

        self._page_size = page_size

    @property
    def pip_libraries(self):
        """Gets the pip_libraries of this RimeListImagesRequest.  # noqa: E501

        Optional. Filters the list for libraries that are installed on the Managed Image. The filter is only active when the list is not empty. When this filter is specified, do not include a pageToken field in the request.  # noqa: E501

        :return: The pip_libraries of this RimeListImagesRequest.  # noqa: E501
        :rtype: list[ListImagesRequestPipLibraryFilter]
        """
        return self._pip_libraries

    @pip_libraries.setter
    def pip_libraries(self, pip_libraries):
        """Sets the pip_libraries of this RimeListImagesRequest.

        Optional. Filters the list for libraries that are installed on the Managed Image. The filter is only active when the list is not empty. When this filter is specified, do not include a pageToken field in the request.  # noqa: E501

        :param pip_libraries: The pip_libraries of this RimeListImagesRequest.  # noqa: E501
        :type: list[ListImagesRequestPipLibraryFilter]
        """

        self._pip_libraries = pip_libraries

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RimeListImagesRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RimeListImagesRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class RegistryGenerativeLanguageModelInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'model_path': 'str',
        'system_prompt': 'str'
    }

    attribute_map = {
        'model_path': 'modelPath',
        'system_prompt': 'systemPrompt'
    }

    def __init__(self, model_path=None, system_prompt=None):  # noqa: E501
        """RegistryGenerativeLanguageModelInfo - a model defined in Swagger"""  # noqa: E501
        self._model_path = None
        self._system_prompt = None
        self.discriminator = None
        self.model_path = model_path
        if system_prompt is not None:
            self.system_prompt = system_prompt

    @property
    def model_path(self):
        """Gets the model_path of this RegistryGenerativeLanguageModelInfo.  # noqa: E501

        The path to the model.  This can be a local path or a path to a file in a configured integration.  # noqa: E501

        :return: The model_path of this RegistryGenerativeLanguageModelInfo.  # noqa: E501
        :rtype: str
        """
        return self._model_path

    @model_path.setter
    def model_path(self, model_path):
        """Sets the model_path of this RegistryGenerativeLanguageModelInfo.

        The path to the model.  This can be a local path or a path to a file in a configured integration.  # noqa: E501

        :param model_path: The model_path of this RegistryGenerativeLanguageModelInfo.  # noqa: E501
        :type: str
        """
        if model_path is None:
            raise ValueError("Invalid value for `model_path`, must not be `None`")  # noqa: E501

        self._model_path = model_path

    @property
    def system_prompt(self):
        """Gets the system_prompt of this RegistryGenerativeLanguageModelInfo.  # noqa: E501

        The prompt to use for the model during each request.  # noqa: E501

        :return: The system_prompt of this RegistryGenerativeLanguageModelInfo.  # noqa: E501
        :rtype: str
        """
        return self._system_prompt

    @system_prompt.setter
    def system_prompt(self, system_prompt):
        """Sets the system_prompt of this RegistryGenerativeLanguageModelInfo.

        The prompt to use for the model during each request.  # noqa: E501

        :param system_prompt: The system_prompt of this RegistryGenerativeLanguageModelInfo.  # noqa: E501
        :type: str
        """

        self._system_prompt = system_prompt

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RegistryGenerativeLanguageModelInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RegistryGenerativeLanguageModelInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from rime_sdk.swagger.swagger_client.api.agent_manager_api import AgentManagerApi
from rime_sdk.swagger.swagger_client.api.config_validator_api import ConfigValidatorApi
from rime_sdk.swagger.swagger_client.api.customer_managed_key_service_api import CustomerManagedKeyServiceApi
from rime_sdk.swagger.swagger_client.api.data_collector_api import DataCollectorApi
from rime_sdk.swagger.swagger_client.api.detection_api import DetectionApi
from rime_sdk.swagger.swagger_client.api.feature_flag_api import FeatureFlagApi
from rime_sdk.swagger.swagger_client.api.file_scanning_api import FileScanningApi
from rime_sdk.swagger.swagger_client.api.file_upload_api import FileUploadApi
from rime_sdk.swagger.swagger_client.api.firewall_api import FirewallApi
from rime_sdk.swagger.swagger_client.api.firewall_instance_manager_api import FirewallInstanceManagerApi
from rime_sdk.swagger.swagger_client.api.firewall_service_api import FirewallServiceApi
from rime_sdk.swagger.swagger_client.api.generative_model_testing_api import GenerativeModelTestingApi
from rime_sdk.swagger.swagger_client.api.image_registry_api import ImageRegistryApi
from rime_sdk.swagger.swagger_client.api.integration_service_api import IntegrationServiceApi
from rime_sdk.swagger.swagger_client.api.job_reader_api import JobReaderApi
from rime_sdk.swagger.swagger_client.api.model_card_service_api import ModelCardServiceApi
from rime_sdk.swagger.swagger_client.api.model_testing_api import ModelTestingApi
from rime_sdk.swagger.swagger_client.api.monitor_service_api import MonitorServiceApi
from rime_sdk.swagger.swagger_client.api.notification_setting_api import NotificationSettingApi
from rime_sdk.swagger.swagger_client.api.project_service_api import ProjectServiceApi
from rime_sdk.swagger.swagger_client.api.rime_info_api import RIMEInfoApi
from rime_sdk.swagger.swagger_client.api.registry_service_api import RegistryServiceApi
from rime_sdk.swagger.swagger_client.api.results_reader_api import ResultsReaderApi
from rime_sdk.swagger.swagger_client.api.schedule_service_api import ScheduleServiceApi
from rime_sdk.swagger.swagger_client.api.security_db_api import SecurityDBApi
from rime_sdk.swagger.swagger_client.api.user_api import UserApi
from rime_sdk.swagger.swagger_client.api.validation_service_api import ValidationServiceApi
from rime_sdk.swagger.swagger_client.api.workspace_service_api import WorkspaceServiceApi



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class TestrunModelProfiling(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'nrows_for_summary': 'str',
        'nrows_for_feature_importance': 'str',
        'metric_configs_json': 'str',
        'impact_metric': 'str',
        'impact_label_threshold': 'float',
        'drift_impact_metric': 'str',
        'subset_summary_metric': 'str',
        'num_feats_for_subset_summary': 'str',
        'custom_metrics': 'list[TestrunCustomMetric]'
    }

    attribute_map = {
        'nrows_for_summary': 'nrowsForSummary',
        'nrows_for_feature_importance': 'nrowsForFeatureImportance',
        'metric_configs_json': 'metricConfigsJson',
        'impact_metric': 'impactMetric',
        'impact_label_threshold': 'impactLabelThreshold',
        'drift_impact_metric': 'driftImpactMetric',
        'subset_summary_metric': 'subsetSummaryMetric',
        'num_feats_for_subset_summary': 'numFeatsForSubsetSummary',
        'custom_metrics': 'customMetrics'
    }

    def __init__(self, nrows_for_summary=None, nrows_for_feature_importance=None, metric_configs_json=None, impact_metric=None, impact_label_threshold=None, drift_impact_metric=None, subset_summary_metric=None, num_feats_for_subset_summary=None, custom_metrics=None):  # noqa: E501
        """TestrunModelProfiling - a model defined in Swagger"""  # noqa: E501
        self._nrows_for_summary = None
        self._nrows_for_feature_importance = None
        self._metric_configs_json = None
        self._impact_metric = None
        self._impact_label_threshold = None
        self._drift_impact_metric = None
        self._subset_summary_metric = None
        self._num_feats_for_subset_summary = None
        self._custom_metrics = None
        self.discriminator = None
        if nrows_for_summary is not None:
            self.nrows_for_summary = nrows_for_summary
        if nrows_for_feature_importance is not None:
            self.nrows_for_feature_importance = nrows_for_feature_importance
        if metric_configs_json is not None:
            self.metric_configs_json = metric_configs_json
        if impact_metric is not None:
            self.impact_metric = impact_metric
        if impact_label_threshold is not None:
            self.impact_label_threshold = impact_label_threshold
        if drift_impact_metric is not None:
            self.drift_impact_metric = drift_impact_metric
        if subset_summary_metric is not None:
            self.subset_summary_metric = subset_summary_metric
        if num_feats_for_subset_summary is not None:
            self.num_feats_for_subset_summary = num_feats_for_subset_summary
        if custom_metrics is not None:
            self.custom_metrics = custom_metrics

    @property
    def nrows_for_summary(self):
        """Gets the nrows_for_summary of this TestrunModelProfiling.  # noqa: E501

        Number of rows to perform inference on the model if no predictions.  # noqa: E501

        :return: The nrows_for_summary of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._nrows_for_summary

    @nrows_for_summary.setter
    def nrows_for_summary(self, nrows_for_summary):
        """Sets the nrows_for_summary of this TestrunModelProfiling.

        Number of rows to perform inference on the model if no predictions.  # noqa: E501

        :param nrows_for_summary: The nrows_for_summary of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._nrows_for_summary = nrows_for_summary

    @property
    def nrows_for_feature_importance(self):
        """Gets the nrows_for_feature_importance of this TestrunModelProfiling.  # noqa: E501

        Number of rows to calculate feature importance over.  # noqa: E501

        :return: The nrows_for_feature_importance of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._nrows_for_feature_importance

    @nrows_for_feature_importance.setter
    def nrows_for_feature_importance(self, nrows_for_feature_importance):
        """Sets the nrows_for_feature_importance of this TestrunModelProfiling.

        Number of rows to calculate feature importance over.  # noqa: E501

        :param nrows_for_feature_importance: The nrows_for_feature_importance of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._nrows_for_feature_importance = nrows_for_feature_importance

    @property
    def metric_configs_json(self):
        """Gets the metric_configs_json of this TestrunModelProfiling.  # noqa: E501

        JSON map of metric API names to keyword arguments, which allows configuration of arbitrary metrics.  # noqa: E501

        :return: The metric_configs_json of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._metric_configs_json

    @metric_configs_json.setter
    def metric_configs_json(self, metric_configs_json):
        """Sets the metric_configs_json of this TestrunModelProfiling.

        JSON map of metric API names to keyword arguments, which allows configuration of arbitrary metrics.  # noqa: E501

        :param metric_configs_json: The metric_configs_json of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._metric_configs_json = metric_configs_json

    @property
    def impact_metric(self):
        """Gets the impact_metric of this TestrunModelProfiling.  # noqa: E501

        Default impact metric.  # noqa: E501

        :return: The impact_metric of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._impact_metric

    @impact_metric.setter
    def impact_metric(self, impact_metric):
        """Sets the impact_metric of this TestrunModelProfiling.

        Default impact metric.  # noqa: E501

        :param impact_metric: The impact_metric of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._impact_metric = impact_metric

    @property
    def impact_label_threshold(self):
        """Gets the impact_label_threshold of this TestrunModelProfiling.  # noqa: E501

        Specifies the threshold for measuring model impact using labeled performance metrics instead of prediction metrics, assuming partial labels.  # noqa: E501

        :return: The impact_label_threshold of this TestrunModelProfiling.  # noqa: E501
        :rtype: float
        """
        return self._impact_label_threshold

    @impact_label_threshold.setter
    def impact_label_threshold(self, impact_label_threshold):
        """Sets the impact_label_threshold of this TestrunModelProfiling.

        Specifies the threshold for measuring model impact using labeled performance metrics instead of prediction metrics, assuming partial labels.  # noqa: E501

        :param impact_label_threshold: The impact_label_threshold of this TestrunModelProfiling.  # noqa: E501
        :type: float
        """

        self._impact_label_threshold = impact_label_threshold

    @property
    def drift_impact_metric(self):
        """Gets the drift_impact_metric of this TestrunModelProfiling.  # noqa: E501

        Default drift impact metric.  # noqa: E501

        :return: The drift_impact_metric of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._drift_impact_metric

    @drift_impact_metric.setter
    def drift_impact_metric(self, drift_impact_metric):
        """Sets the drift_impact_metric of this TestrunModelProfiling.

        Default drift impact metric.  # noqa: E501

        :param drift_impact_metric: The drift_impact_metric of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._drift_impact_metric = drift_impact_metric

    @property
    def subset_summary_metric(self):
        """Gets the subset_summary_metric of this TestrunModelProfiling.  # noqa: E501

        The subset performance degradation summary metric is calculated by taking the difference between the worst subset degradation and the overall degradation of the configured metric.  # noqa: E501

        :return: The subset_summary_metric of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._subset_summary_metric

    @subset_summary_metric.setter
    def subset_summary_metric(self, subset_summary_metric):
        """Sets the subset_summary_metric of this TestrunModelProfiling.

        The subset performance degradation summary metric is calculated by taking the difference between the worst subset degradation and the overall degradation of the configured metric.  # noqa: E501

        :param subset_summary_metric: The subset_summary_metric of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._subset_summary_metric = subset_summary_metric

    @property
    def num_feats_for_subset_summary(self):
        """Gets the num_feats_for_subset_summary of this TestrunModelProfiling.  # noqa: E501

        Number of features over which the subset performance degradation summary metric is aggregated.  # noqa: E501

        :return: The num_feats_for_subset_summary of this TestrunModelProfiling.  # noqa: E501
        :rtype: str
        """
        return self._num_feats_for_subset_summary

    @num_feats_for_subset_summary.setter
    def num_feats_for_subset_summary(self, num_feats_for_subset_summary):
        """Sets the num_feats_for_subset_summary of this TestrunModelProfiling.

        Number of features over which the subset performance degradation summary metric is aggregated.  # noqa: E501

        :param num_feats_for_subset_summary: The num_feats_for_subset_summary of this TestrunModelProfiling.  # noqa: E501
        :type: str
        """

        self._num_feats_for_subset_summary = num_feats_for_subset_summary

    @property
    def custom_metrics(self):
        """Gets the custom_metrics of this TestrunModelProfiling.  # noqa: E501

        List of custom metrics.  # noqa: E501

        :return: The custom_metrics of this TestrunModelProfiling.  # noqa: E501
        :rtype: list[TestrunCustomMetric]
        """
        return self._custom_metrics

    @custom_metrics.setter
    def custom_metrics(self, custom_metrics):
        """Sets the custom_metrics of this TestrunModelProfiling.

        List of custom metrics.  # noqa: E501

        :param custom_metrics: The custom_metrics of this TestrunModelProfiling.  # noqa: E501
        :type: list[TestrunCustomMetric]
        """

        self._custom_metrics = custom_metrics

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TestrunModelProfiling, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TestrunModelProfiling):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class TestrunresultResultSummaryCounts(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'total': 'str',
        '_pass': 'str',
        'warning': 'str',
        'fail': 'str',
        'skip': 'str'
    }

    attribute_map = {
        'total': 'total',
        '_pass': 'pass',
        'warning': 'warning',
        'fail': 'fail',
        'skip': 'skip'
    }

    def __init__(self, total=None, _pass=None, warning=None, fail=None, skip=None):  # noqa: E501
        """TestrunresultResultSummaryCounts - a model defined in Swagger"""  # noqa: E501
        self._total = None
        self.__pass = None
        self._warning = None
        self._fail = None
        self._skip = None
        self.discriminator = None
        if total is not None:
            self.total = total
        if _pass is not None:
            self._pass = _pass
        if warning is not None:
            self.warning = warning
        if fail is not None:
            self.fail = fail
        if skip is not None:
            self.skip = skip

    @property
    def total(self):
        """Gets the total of this TestrunresultResultSummaryCounts.  # noqa: E501

        The total number of tests.  # noqa: E501

        :return: The total of this TestrunresultResultSummaryCounts.  # noqa: E501
        :rtype: str
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this TestrunresultResultSummaryCounts.

        The total number of tests.  # noqa: E501

        :param total: The total of this TestrunresultResultSummaryCounts.  # noqa: E501
        :type: str
        """

        self._total = total

    @property
    def _pass(self):
        """Gets the _pass of this TestrunresultResultSummaryCounts.  # noqa: E501

        The number of tests that pass.  # noqa: E501

        :return: The _pass of this TestrunresultResultSummaryCounts.  # noqa: E501
        :rtype: str
        """
        return self.__pass

    @_pass.setter
    def _pass(self, _pass):
        """Sets the _pass of this TestrunresultResultSummaryCounts.

        The number of tests that pass.  # noqa: E501

        :param _pass: The _pass of this TestrunresultResultSummaryCounts.  # noqa: E501
        :type: str
        """

        self.__pass = _pass

    @property
    def warning(self):
        """Gets the warning of this TestrunresultResultSummaryCounts.  # noqa: E501

        The number of tests that raise a warning.  # noqa: E501

        :return: The warning of this TestrunresultResultSummaryCounts.  # noqa: E501
        :rtype: str
        """
        return self._warning

    @warning.setter
    def warning(self, warning):
        """Sets the warning of this TestrunresultResultSummaryCounts.

        The number of tests that raise a warning.  # noqa: E501

        :param warning: The warning of this TestrunresultResultSummaryCounts.  # noqa: E501
        :type: str
        """

        self._warning = warning

    @property
    def fail(self):
        """Gets the fail of this TestrunresultResultSummaryCounts.  # noqa: E501

        The number of tests that fail.  # noqa: E501

        :return: The fail of this TestrunresultResultSummaryCounts.  # noqa: E501
        :rtype: str
        """
        return self._fail

    @fail.setter
    def fail(self, fail):
        """Sets the fail of this TestrunresultResultSummaryCounts.

        The number of tests that fail.  # noqa: E501

        :param fail: The fail of this TestrunresultResultSummaryCounts.  # noqa: E501
        :type: str
        """

        self._fail = fail

    @property
    def skip(self):
        """Gets the skip of this TestrunresultResultSummaryCounts.  # noqa: E501

        The number of tests that are skipped.  # noqa: E501

        :return: The skip of this TestrunresultResultSummaryCounts.  # noqa: E501
        :rtype: str
        """
        return self._skip

    @skip.setter
    def skip(self, skip):
        """Sets the skip of this TestrunresultResultSummaryCounts.

        The number of tests that are skipped.  # noqa: E501

        :param skip: The skip of this TestrunresultResultSummaryCounts.  # noqa: E501
        :type: str
        """

        self._skip = skip

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TestrunresultResultSummaryCounts, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TestrunresultResultSummaryCounts):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Robust Intelligence REST API

    API methods for Robust Intelligence. Users must authenticate using the `X-Firewall-Api-Key` for all the firewall methods and `rime-api-key` for all other methods.  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: dev@robustintelligence.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class NotificationDigestConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'frequency': 'DigestConfigDigestFrequency',
        'hour_offset': 'int',
        'last_digest_time': 'datetime'
    }

    attribute_map = {
        'frequency': 'frequency',
        'hour_offset': 'hourOffset',
        'last_digest_time': 'lastDigestTime'
    }

    def __init__(self, frequency=None, hour_offset=None, last_digest_time=None):  # noqa: E501
        """NotificationDigestConfig - a model defined in Swagger"""  # noqa: E501
        self._frequency = None
        self._hour_offset = None
        self._last_digest_time = None
        self.discriminator = None
        if frequency is not None:
            self.frequency = frequency
        if hour_offset is not None:
            self.hour_offset = hour_offset
        if last_digest_time is not None:
            self.last_digest_time = last_digest_time

    @property
    def frequency(self):
        """Gets the frequency of this NotificationDigestConfig.  # noqa: E501


        :return: The frequency of this NotificationDigestConfig.  # noqa: E501
        :rtype: DigestConfigDigestFrequency
        """
        return self._frequency

    @frequency.setter
    def frequency(self, frequency):
        """Sets the frequency of this NotificationDigestConfig.


        :param frequency: The frequency of this NotificationDigestConfig.  # noqa: E501
        :type: DigestConfigDigestFrequency
        """

        self._frequency = frequency

    @property
    def hour_offset(self):
        """Gets the hour_offset of this NotificationDigestConfig.  # noqa: E501

        The offset in the day when the digest starts. The offset is taken in the configured timezone for the workspace. Uses a default value of 08:00 when not provided.  # noqa: E501

        :return: The hour_offset of this NotificationDigestConfig.  # noqa: E501
        :rtype: int
        """
        return self._hour_offset

    @hour_offset.setter
    def hour_offset(self, hour_offset):
        """Sets the hour_offset of this NotificationDigestConfig.

        The offset in the day when the digest starts. The offset is taken in the configured timezone for the workspace. Uses a default value of 08:00 when not provided.  # noqa: E501

        :param hour_offset: The hour_offset of this NotificationDigestConfig.  # noqa: E501
        :type: int
        """

        self._hour_offset = hour_offset

    @property
    def last_digest_time(self):
        """Gets the last_digest_time of this NotificationDigestConfig.  # noqa: E501

        The last time that the RI platform sent the digest. This timestamp is zero when no digests have ever been sent.  # noqa: E501

        :return: The last_digest_time of this NotificationDigestConfig.  # noqa: E501
        :rtype: datetime
        """
        return self._last_digest_time

    @last_digest_time.setter
    def last_digest_time(self, last_digest_time):
        """Sets the last_digest_time of this NotificationDigestConfig.

        The last time that the RI platform sent the digest. This timestamp is zero when no digests have ever been sent.  # noqa: E501

        :param last_digest_time: The last_digest_time of this NotificationDigestConfig.  # noqa: E501
        :type: datetime
        """

        self._last_digest_time = last_digest_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NotificationDigestConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NotificationDigestConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Library defining the interfaces for monitoring jobs in the RIME backend."""

import re
import time
from abc import ABC, abstractmethod
from datetime import datetime
from http import HTTPStatus
from typing import Any, Dict, List, Optional

from rime_sdk.internal.rest_error_handler import RESTErrorHandler
from rime_sdk.swagger import swagger_client
from rime_sdk.swagger.swagger_client import ApiClient
from rime_sdk.swagger.swagger_client.models import (
    RimeJobData,
    RimeJobMetadata,
    RimeJobType,
    RimeJobView,
    RimeTestRunProgress,
    RimeTestTaskStatus,
    StatedbJobStatus,
)
from rime_sdk.swagger.swagger_client.rest import ApiException
from rime_sdk.test_run import TestRun

CANCELLABLE_JOB_TYPES = [
    RimeJobType.FILE_SCAN,
    RimeJobType.FIREWALL_BATCH_TEST,
    RimeJobType.MODEL_STRESS_TEST,
]
# Format datetimes in user-readable format
DATETIME_FIELDS = ["creation_time", "start_time", "completion_time"]
# Removed if empty or overly verbose
FIELDS_TO_BE_REMOVED = ["job_data", "archived_job_logs"]


class BaseJob(ABC):
    """Abstract class for RIME Jobs.

    This object provides an interface for monitoring the status of a
    job in the RIME backend.
    """

    _job_type: str

    def __init__(self, api_client: ApiClient, job_id: str) -> None:
        """Create a new RIME Job.

        Args:
            api_client: ApiClient
                The client used to query the RIME cluster.
            job_id: str
                The ID of the RIME Job object.
        """
        self._api_client = api_client
        self._job_id = job_id

    def __repr__(self) -> str:
        """Return a string representation of this object."""
        return str(self)

    @property
    def job_id(self) -> str:
        """Return the ID of the Job."""
        return self._job_id

    @property
    def job_type(self) -> str:
        """Return the type of the Job.

        The valid Job types are:
        MODEL_STRESS_TEST, FIREWALL_BATCH_TEST, IMAGE_BUILDER, FILE_SCAN
        """
        return self._job_type

    @property
    def error_msg(self) -> str:
        """The error message, if this job failed."""
        status = self.get_status(verbose=False)
        return f"{status.get('error_msg')}"

    def __str__(self) -> str:
        """Pretty-print the object."""
        # Ping the backend for the full representation of the job and detailed
        # info only for succeeded jobs.
        job = self.get_status(verbose=False)
        ret = {
            "job_id": self._job_id,
            "job_type": self._job_type,
            "status": job.get("status"),
        }
        if job.get("status") == StatedbJobStatus.SUCCEEDED:
            ret.update(self._get_test_run_dict())
        elif job.get("status") == StatedbJobStatus.FAILED:
            ret["error_msg"] = job.get("error_msg")
        job_class = self.__class__.__name__
        return f"{job_class} {ret}"

    def __eq__(self, obj: Any) -> bool:
        """Check if this job is equivalent to 'obj'."""
        return isinstance(obj, BaseJob) and self._job_id == obj._job_id

    @staticmethod
    def _get_test_run_progress(test_run: RimeTestRunProgress) -> Optional[str]:
        test_batches = test_run.test_batches
        if len(test_batches) == 0:
            return None
        n = sum(batch.status == RimeTestTaskStatus.COMPLETED for batch in test_batches)
        return f"{n:<2} / {len(test_batches):>2} tests completed"

    @abstractmethod
    def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
        """Pretty print the progress of the test run."""

    # TODO(VAL-2282): refactor this function to have fewer branches and be more readable
    def get_status(  # noqa: PLR0912 (too-many-branches)
        self,
        verbose: bool = True,
        wait_until_finish: bool = False,
        poll_rate_sec: float = 5.0,
    ) -> Dict:
        """Return the current status of the Job.

        Includes flags for blocking until the job is complete and printing
        information to ``stdout``. This method displays the progress and
        running time of jobs.

        If the job has failed, the logs of the testing engine will be dumped
        to ``stdout`` for debugging.

        Arguments:
            verbose: bool
                Specifies whether to print diagnostic information such as progress.
            wait_until_finish: bool
                Specifies whether to block until the job has succeeded or failed.
                If `verbose` is enabled too, information about the job including
                running time and progress will be printed to ``stdout`` every
                ``poll_rate_sec``.
            poll_rate_sec: float
                The frequency in seconds with which to poll the job's status.

        Returns:
            Dict:
                A dictionary representing the job's state.

                .. code-block:: python

                    {
                    "id": str
                    "type": str
                    "status": str
                    "start_time_secs": int64
                    "running_time_secs": double
                    }

        Example:
            .. code-block:: python

                # Block until this job is finished and dump monitoring info to stdout.
                job_status = job.get_status(verbose=True, wait_until_finish=True)
        """
        printed_cancellation_req = False
        printed_poll_string = False
        # Create backend client stubs to use for the remainder of this session.
        api = swagger_client.JobReaderApi(self._api_client)
        with RESTErrorHandler():
            job: RimeJobMetadata = api.job_reader_get_job(
                job_id=self._job_id, view=RimeJobView.FULL
            ).job
        if verbose:
            print(
                "Job '{}' started at {}".format(
                    job.job_id,
                    datetime.fromtimestamp(job.creation_time.timestamp()),
                )
            )

        # Do not repeat if the job is finished or blocking is disabled.
        while wait_until_finish and job.status not in (
            StatedbJobStatus.SUCCEEDED,
            StatedbJobStatus.FAILED,
            StatedbJobStatus.CANCELLED,
        ):
            time.sleep(poll_rate_sec)
            with RESTErrorHandler():
                job = api.job_reader_get_job(
                    job_id=self._job_id, view=RimeJobView.FULL
                ).job
                progress = self._get_progress(job)
            if verbose:
                if (
                    not printed_cancellation_req
                    and verbose
                    and job.cancellation_requested
                ):
                    print("Cancellation Requested")
                    # Only print "Cancellation Requested" once.
                    printed_cancellation_req = True

                minute, second = divmod(job.running_time_secs, 60)
                hour, minute = divmod(minute, 60)
                progress_str = f" ({progress})" if progress else ""
                print(
                    "\rStatus: {}, Running Time: {:02}:{:02}:{:05.2f}{}".format(
                        job.status, int(hour), int(minute), second, progress_str
                    ),
                    end="",
                )
                printed_poll_string = True

        # Add a new line if we printed the poll string, because it has `end=""`.
        # This ensures future print statements start on the next line.
        if verbose and printed_poll_string:
            print()

        # Only get the logs if the job has failed, as the
        # primary purpose is debuggability during development.
        model_testing_api = swagger_client.ModelTestingApi(self._api_client)
        if job.status == StatedbJobStatus.FAILED:
            if job.error_msg is not None:
                print(f"Error message from failed job: {job.error_msg}")

            try:
                response = model_testing_api.model_testing_get_latest_logs(
                    job_id=self._job_id
                )
                if response.result.chunk:
                    print(
                        "\nLogs from failed job:\n"
                        + self._format_logs(response.result.chunk)
                    )
                else:
                    print("No logs found.")
            except ApiException as e:
                # if logs cannot be found do not raise an error.
                # For most deployments logs cannot be read via this api.
                if e.status != HTTPStatus.NOT_FOUND:
                    raise ValueError(e.reason) from None
            # check for debug logs in the job
            print(f"Debug Logs: {self._get_debug_logs(job)}")

        job_dict = job.to_dict()
        # Hide job data and archived logs in the return value because it can get ugly.
        for field in FIELDS_TO_BE_REMOVED:
            if field in job_dict:
                del job_dict[field]

        for field in DATETIME_FIELDS:
            if field in job_dict and job_dict[field]:
                job_dict[field] = job_dict[field].strftime("%H:%M %B %d, %Y")

        return job_dict

    @staticmethod
    def _format_logs(logs_string: str) -> str:
        # Required to remove any warnings which contain endpoint information from logs.
        logs_without_warning = re.sub(r"^WARNING(.*?)recommended!\n$", "", logs_string)
        return logs_without_warning

    def _get_test_run_dict(self) -> dict:
        """Return a dictionary that is used to help pretty-print this object."""
        return {}

    def cancel(self) -> None:
        """Request to cancel the Job.

        The RIME cluster will mark the Job with "Cancellation Requested" and then clean
        up the Job.
        """
        if self.job_type not in CANCELLABLE_JOB_TYPES:
            raise ValueError(
                "Cancelling jobs is only supported for job types "
                f"{CANCELLABLE_JOB_TYPES}"
            )
        api = swagger_client.JobReaderApi(self._api_client)
        with RESTErrorHandler():
            api.job_reader_cancel_job(job_id=self.job_id)

    def get_agent_id(self) -> str:
        """Return the Agent ID running the Job."""
        api = swagger_client.JobReaderApi(self._api_client)
        with RESTErrorHandler():
            job = api.job_reader_get_job(job_id=self._job_id).job
        return job.agent_id.uuid

    def _get_job(self) -> RimeJobData:
        """Return the Job object.

        Get the Test Run ID corresponding to a successful Job.

        Raises:
            ValueError
                This error is generated when the job does not have state 'SUCCEEDED'
                or when the request to the Job Reader service fails.
        """
        # This first step only prevents a rare case where the RIME engine has
        # signaled the test suite has completed but before the upload has completed.
        api = swagger_client.JobReaderApi(self._api_client)
        with RESTErrorHandler():
            job: RimeJobMetadata = api.job_reader_get_job(job_id=self._job_id).job
        if job.status != StatedbJobStatus.SUCCEEDED:
            raise ValueError(
                f"Job has status {job.status}; "
                "it must have status {StatedbJobStatus.SUCCEEDED} to get results"
            )
        return job.job_data

    def get_job_debug_logs_link(self) -> str:
        """Return a link to the archived logs for a failed Job."""
        api = swagger_client.JobReaderApi(self._api_client)
        with RESTErrorHandler():
            job: RimeJobMetadata = api.job_reader_get_job(
                job_id=self._job_id, view=RimeJobView.FULL
            ).job
            return self._get_debug_logs(job)

    def _get_debug_logs(self, job: RimeJobMetadata) -> str:
        """Pretty print the archived logs for a failed Job."""
        if job.status != StatedbJobStatus.FAILED:
            return (
                f"Debug logs are not available for this job."
                f"They are only available for failed jobs, "
                f"this job has status {job.status}"
            )
        else:
            try:
                expiration_time = datetime.fromtimestamp(
                    job.archived_job_logs.expiration_time.timestamp()
                )
                if datetime.now() < expiration_time:
                    return (
                        f"Debug logs for the job can found in your blob "
                        f"storage for your job {job.job_id},"
                        f" at the following url {job.archived_job_logs.url}"
                    )
                else:
                    return (
                        f"Debug logs for the job {job.job_id} maybe "
                        f"available in your blob storage. RI platform only"
                        f" has access to a presigned url which has "
                        f"expired. However, the logs themselves might not "
                        f"have expired. Please check your configured blob"
                        f" storage. The logs maybe available in the "
                        f"bucket configured under the following directory:"
                        f" /logs/{job.job_id}. The expired url:"
                        f"{job.archived_job_logs.url}"
                    )
            except AttributeError:
                return "Debug logs are not available for this job"


class Job(BaseJob):
    """This object provides an interface for monitoring a Stress Test Job in the RIME backend."""

    _job_type = RimeJobType.MODEL_STRESS_TEST

    def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
        """Pretty print the progress of the test run."""
        if job.job_data.stress.progress is None:
            return None
        else:
            return self._get_test_run_progress(job.job_data.stress.progress.test_run)

    def get_test_run_id(self) -> str:
        """Get the Test Run ID corresponding to a successful Job.

        Raises:
            ValueError
                This error is generated when the job does not have state 'SUCCEEDED'
                or when the request to the Job Reader service fails.
        """
        job_data = self._get_job()
        return job_data.stress.test_run_id

    def get_test_run(self) -> TestRun:
        """Get the Test Run object corresponding to a successful Job.

        Raises:
            ValueError
                This error is generated when the job does not have state 'SUCCEEDED'
                or if the request to the Job Reader service fails.
        """
        test_run_id = self.get_test_run_id()
        return TestRun(self._api_client, test_run_id)

    def _get_test_run_dict(self) -> dict:
        """Return a dictionary that is used to help pretty-print this object."""
        ret = {}
        test_run_obj = self.get_test_run()
        test_run_res = test_run_obj.get_result_df()
        for k, v in test_run_res.iloc[0].to_dict().items():
            # Omit metrics from the output dictionary.
            if not str(k).startswith("metrics"):
                ret[k] = v
        return ret


class ContinuousTestJob(BaseJob):
    """This object provides an interface for monitoring a Continuous Test Job in the RIME backend."""

    _job_type = RimeJobType.FIREWALL_BATCH_TEST

    def _get_progress(self, job: RimeJobMetadata) -> Optional[str]:
        """Pretty print the progress of the test run."""
        if job.job_data.continuous_inc.progress is None:
            return None
        test_runs = job.job_data.continuous_inc.progress.test_runs
        total_ct_runs = len(job.job_data.continuous_inc.ct_test_run_ids)
        if total_ct_runs == 0:
            return None
        finished_test_runs = len(
            [
                test_run
                for test_run in test_runs
                if test_run.test_run.status == RimeTestTaskStatus.COMPLETED
            ]
        )
        run_progress = "{:<2} / {:>2} time bins completed".format(
            finished_test_runs, total_ct_runs
        )
        for test_run in test_runs:
            if test_run.test_run.status != RimeTestTaskStatus.COMPLETED:
                batch_progress = self._get_test_run_progress(test_run.test_run)
                if batch_progress is not None:
                    return f"{run_progress}, {batch_progress}"
        return run_progress

    def get_test_run_ids(self) -> str:
        """Get the set of Test Run IDs corresponding to a successful Continuous Test Job.

        Raises:
            ValueError
                This error is generated when the job does not have state 'SUCCEEDED'
                or when the request to the Job Reader service fails.
        """
        job_data = self._get_job()
        return job_data.continuous_inc.ct_test_run_ids

    def get_test_runs(self) -> List[TestRun]:
        """Get the list of Test Run objects corresponding to a successful Continuous Test Job.

        Raises:
            ValueError
                This error is generated when the job does not have state 'SUCCEEDED'
                or if the request to the Job Reader service fails.
        """
        test_run_ids = self.get_test_run_ids()
        return [TestRun(self._api_client, test_run_id) for test_run_id in test_run_ids]

    def _get_test_run_dict(self) -> dict:
        """Return a dictionary that is used to help pretty-print this object."""
        return {}


class ImageBuilderJob(BaseJob):
    """This object provides an interface for monitoring a Image Builder Job in the RIME backend."""

    _job_type = RimeJobType.IMAGE_BUILDER

    def _get_progress(
        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
    ) -> Optional[str]:
        """Pretty print the progress of the test run."""
        # TODO: find a good way to pretty print the progress of a ImageBuilderJob
        return None


class FileScanJob(BaseJob):
    """This object provides an interface for monitoring a File Scan Job in the RIME backend."""

    _job_type = RimeJobType.FILE_SCAN

    def _get_progress(
        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
    ) -> Optional[str]:
        """Pretty print the progress of the test run."""
        # TODO: find a good way to pretty print the progress of a FileScanJob
        return None


class GenerativeModelTestJob(BaseJob):
    """This object provides an interface for monitoring a Generative Model Test Job in the RIME backend."""

    _job_type = RimeJobType.GENERATIVE_MODEL_TEST

    def _get_progress(
        self, job: RimeJobMetadata  # noqa: ARG002 (unused-method-argument)
    ) -> Optional[str]:
        """Pretty print the progress of the test run."""
        return None



"""Library for internal helpers on test objects."""
from http import HTTPStatus
from typing import Optional

from rime_sdk.swagger import swagger_client
from rime_sdk.swagger.swagger_client import ApiClient
from rime_sdk.swagger.swagger_client.models import TestrunresultGetBatchResultResponse
from rime_sdk.swagger.swagger_client.rest import ApiException


def get_batch_result_response(
    test_run_id: str, test_type: str, api_client: Optional[ApiClient] = None
) -> TestrunresultGetBatchResultResponse:
    """Obtain the test batch summary response."""
    api = swagger_client.ResultsReaderApi(api_client)
    try:
        res: TestrunresultGetBatchResultResponse = api.results_reader_get_batch_result(
            test_run_id=test_run_id, test_type=test_type
        )
        return res
    except ApiException as e:
        if e.status == HTTPStatus.NOT_FOUND:
            raise ValueError(
                f"The test batch for {test_type} and test run "
                f"{test_run_id} was not found."
            ) from None
        raise ValueError(e.reason) from None
